@@ -6,29 +6,54 @@ import sympy
 from functools import total_ordering
 import importlib
 from itertools import chain
-import typing
-from typing import Any, cast
+from typing import Any, Callable, Dict, Iterable, List, Optional, Union, cast
 
-from mathics.version import __version__  # noqa used in loading to check consistency.
 
-from mathics.core.convert import from_sympy
-from mathics.core.definitions import Definition
-from mathics.core.parser.util import SystemDefinitions, PyMathicsDefinitions
-from mathics.core.rules import Rule, BuiltinRule, Pattern
-from mathics.core.expression import (
-    BaseExpression,
-    Expression,
+from mathics.builtin.exceptions import (
+    BoxConstructError,
+    MessageException,
+)
+
+from mathics.core.atoms import (
     Integer,
     MachineReal,
     PrecisionReal,
     String,
+)
+from mathics.core.attributes import protected, read_protected, no_attributes
+from mathics.core.convert.expression import to_expression
+from mathics.core.convert.python import from_bool
+from mathics.core.convert.sympy import from_sympy
+from mathics.core.definitions import Definition
+from mathics.core.element import BoxElement
+from mathics.core.expression import Expression, SymbolDefault
+from mathics.core.list import ListExpression
+from mathics.core.number import get_precision, PrecisionValueError
+from mathics.core.parser.util import SystemDefinitions, PyMathicsDefinitions
+from mathics.core.rules import Rule, BuiltinRule, Pattern
+from mathics.core.symbols import (
+    BaseElement,
     Symbol,
-    SymbolFalse,
-    SymbolTrue,
+    SymbolHoldForm,
     ensure_context,
     strip_context,
 )
-from mathics.core.numbers import get_precision, PrecisionValueError
+from mathics.core.systemsymbols import SymbolMessageName, SymbolRule
+
+
+def check_requires_list(requires: list) -> bool:
+    """
+    Check if module names in ``requires`` can be imported and return True if they can or False if not.
+    """
+    for package in requires:
+        lib_is_installed = True
+        try:
+            lib_is_installed = importlib.util.find_spec(package) is not None
+        except ImportError:
+            lib_is_installed = False
+        if not lib_is_installed:
+            return False
+    return True
 
 
 def get_option(options, name, evaluation, pop=False, evaluate=True):
@@ -55,23 +80,123 @@ def has_option(options, name, evaluation):
     return get_option(options, name, evaluation, evaluate=False) is not None
 
 
+def split_name(name: str) -> str:
+    """
+    insert spaces in front of upper case letters
+    and numbers. For instance,
+    ``split_name("BezierCurve3D")`` results in
+    ``"bezier curve 3D"``
+
+    """
+    if name == "":
+        return ""
+    result = name[0]
+    for i in range(1, len(name)):
+        if name[i].isupper():
+            if not name[i - 1].isdigit():
+                result = result + " "
+        elif name[i].isdigit():
+            if not name[i - 1].isdigit():
+                result = result + " "
+        result = result + name[i]
+    return result.lower()
+
+
 mathics_to_python = {}
 
 
-class Builtin(object):
-    name: typing.Optional[str] = None
-    context = ""
-    abstract = False
-    attributes: typing.Tuple[Any, ...] = ()
-    rules: typing.Dict[str, Any] = {}
-    formats: typing.Dict[str, Any] = {}
-    messages: typing.Dict[str, Any] = {}
-    options: typing.Dict[str, Any] = {}
+class Builtin:
+    """
+    A base class for a Built-in function symbols, like List, or variables, like $SystemID,
+    and Built-in Objects, like DateTimeObject.
+
+    Some of the class variables of the Builtin object are used to
+    create a definition object for that built-in symbol.  In particular,
+    there are (transformation) rules, attributes, (error) messages,
+    options, and other things.
+
+    Function application pattern matching
+    -------------------------------------
+
+    Method names of a builtin-class that start with the word ``apply`` are evaluation methods that
+    will get called when the docstring of that method matches the expression to be evaluated.
+
+    For example:
+
+    ```
+        def apply(x, evaluation):
+             "F[x_Real]"
+             return Expression(Symbol("G"), x*2)
+    ```
+
+    adds a ``BuiltinRule`` to the symbol's definition object that implements ``F[x_]->G[x*2]``.
+
+    As shown in the example above, leading argument names of the
+    function are the arguments mentioned in the names given up to the
+    first underscore ``_``.  So the single parameter in the above is
+    ``x``. The method must also have an evaluation parameter, and may
+    have an optional `options` parameter.
+
+    If the ``apply*`` method returns ``None``, the replacement fails, and the expression keeps its original form.
+
+    For rules including ``OptionsPattern``
+    ```
+        def apply_with_options(x, evaluation, options):
+             '''F[x_Real, OptionsPattern[]]'''
+             ...
+    ```
+    the options are stored as a dictionary in the last parameter. For example, if the rule is applied to ``F[x, Method->Automatic]``
+    the expression is replaced by the output of ``apply_with_options(x, evaluation, {"System`Method": Symbol("Automatic")})
+
+    The method ``contribute`` stores the definition of the  ``Builtin`` ` `Symbol`` into a set of ``Definitions``. For example,
+
+    ```
+    definitions = Definitions(add_builtin=False)
+    List(expression=False).contribute(definitions)
+    ```
+    produces a ``Definitions`` object with just one definition, for the ``Symbol`` ``System`List``.
+
+    Notice that for creating a Builtin, we must pass to the constructor the option ``expression=False``. Otherwise,
+    an Expression object is created, with the ``Symbol`` associated to the definition as the ``Head``.
+    For example,
+
+    ```
+    builtinlist = List(expression=False)
+    ```
+    creates the  ``Builtin``  ``List``, associated to the symbol ``System`List``, but
+
+    ```
+    expr_list = List(Integer(1), Integer(2), Integer(3))
+    ```
+    is equivalent to:
+    ```
+    expr_list = ListExpression(Integer(1), Integer(2), Integer(3))
+    ```
+    """
+
+    name: Optional[str] = None
+    context: str = ""
+    abstract: bool = False
+    attributes: int = protected
+    is_numeric: bool = False
+    rules: Dict[str, Any] = {}
+    formats: Dict[str, Any] = {}
+    messages: Dict[str, Any] = {}
+    options: Dict[str, Any] = {}
     defaults = {}
 
     def __new__(cls, *args, **kwargs):
+        # comment @mmatera:
+        # The goal of this method is to allow to build expressions
+        # like ``ListExpression(x,y,z)``
+        # in the handy way  ``List(x,y,z)``.
+        # This is handy, but can be confusing if this is not very
+        # well documented.
+        # Notice that this behavior was used extensively in
+        # mathics.builtin.inout
+
         if kwargs.get("expression", None) is not False:
-            return Expression(cls.get_name(), *args)
+            return to_expression(cls.get_name(), *args)
         else:
             instance = super().__new__(cls)
             if not instance.formats:
@@ -107,9 +232,7 @@ class Builtin(object):
                 # Otherwise it'll be created in Global` when it's
                 # used, so it won't work.
                 if option not in definitions.builtin:
-                    definitions.builtin[option] = Definition(
-                        name=name, attributes=set()
-                    )
+                    definitions.builtin[option] = Definition(name=name)
 
         # Check if the given options are actually supported by the Builtin.
         # If not, we might issue an optx error and abort. Using '$OptionSyntax'
@@ -130,7 +253,7 @@ class Builtin(object):
                         evaluation.message(
                             name,
                             "optx",
-                            Expression("Rule", short_key, value),
+                            Expression(SymbolRule, String(short_key), value),
                             strip_context(name),
                         )
                         if option_syntax in ("Strict", "System`Strict"):
@@ -154,7 +277,7 @@ class Builtin(object):
                 BuiltinRule(name, pattern, function, check_options, system=True)
             )
         for pattern, replace in self.rules.items():
-            if not isinstance(pattern, BaseExpression):
+            if not isinstance(pattern, BaseElement):
                 pattern = pattern % {"name": name}
                 pattern = parse_builtin_rule(pattern, definition_class)
             replace = replace % {"name": name}
@@ -206,7 +329,7 @@ class Builtin(object):
             for form in forms:
                 if form not in formatvalues:
                     formatvalues[form] = []
-                if not isinstance(pattern, BaseExpression):
+                if not isinstance(pattern, BaseElement):
                     pattern = pattern % {"name": name}
                     pattern = parse_builtin_rule(pattern)
                 replace = replace % {"name": name}
@@ -216,9 +339,11 @@ class Builtin(object):
         for form, formatrules in formatvalues.items():
             formatrules.sort()
 
+        if hasattr(self, "summary_text"):
+            self.messages["usage"] = self.summary_text
         messages = [
             Rule(
-                Expression("MessageName", Symbol(name), String(msg)),
+                Expression(SymbolMessageName, Symbol(name), String(msg)),
                 String(value),
                 system=True,
             )
@@ -227,20 +352,12 @@ class Builtin(object):
 
         messages.append(
             Rule(
-                Expression("MessageName", Symbol(name), String("optx")),
+                Expression(SymbolMessageName, Symbol(name), String("optx")),
                 String("`1` is not a supported option for `2`[]."),
                 system=True,
             )
         )
 
-        if "Unprotected" in self.attributes:
-            attributes = []
-            self.attributes = list(self.attributes)
-            self.attributes.remove("Unprotected")
-        else:
-            attributes = ["System`Protected"]
-
-        attributes += list(ensure_context(a) for a in self.attributes)
         options = {}
         for option, value in self.options.items():
             option = ensure_context(option)
@@ -250,17 +367,15 @@ class Builtin(object):
                 # Otherwise it'll be created in Global` when it's
                 # used, so it won't work.
                 if option not in definitions.builtin:
-                    definitions.builtin[option] = Definition(
-                        name=name, attributes=set()
-                    )
+                    definitions.builtin[option] = Definition(name=name)
         defaults = []
         for spec, value in self.defaults.items():
             value = parse_builtin_rule(value)
             pattern = None
             if spec is None:
-                pattern = Expression("Default", Symbol(name))
+                pattern = Expression(SymbolDefault, Symbol(name))
             elif isinstance(spec, int):
-                pattern = Expression("Default", Symbol(name), Integer(spec))
+                pattern = Expression(SymbolDefault, Symbol(name), Integer(spec))
             if pattern is not None:
                 defaults.append(Rule(pattern, value, system=True))
 
@@ -269,10 +384,11 @@ class Builtin(object):
             rules=rules,
             formatvalues=formatvalues,
             messages=messages,
-            attributes=attributes,
+            attributes=self.attributes,
             options=options,
             defaultvalues=defaults,
             builtin=self,
+            is_numeric=self.is_numeric,
         )
         if is_pymodule:
             definitions.pymathics[name] = definition
@@ -293,10 +409,10 @@ class Builtin(object):
             return shortname
         return cls.context + shortname
 
-    def get_operator(self) -> typing.Optional[str]:
+    def get_operator(self) -> Optional[str]:
         return None
 
-    def get_operator_display(self) -> typing.Optional[str]:
+    def get_operator_display(self) -> Optional[str]:
         return None
 
     def get_functions(self, prefix="apply", is_pymodule=False):
@@ -305,7 +421,6 @@ class Builtin(object):
         unavailable_function = self._get_unavailable_function()
         for name in dir(self):
             if name.startswith(prefix):
-
                 function = getattr(self, name)
                 pattern = function.__doc__
                 if pattern is None:  # Fixes PyPy bug
@@ -337,25 +452,22 @@ class Builtin(object):
     def get_option(options, name, evaluation, pop=False):
         return get_option(options, name, evaluation, pop)
 
-    def _get_unavailable_function(self):
+    def _get_unavailable_function(self) -> Optional[Callable]:
+        """
+        If some of the required libraries for a symbol are not available,
+        returns a default function that override the ``apply_`` methods
+        of the class. Otherwise, returns ``None``.
+        """
+
+        def apply_unavailable(**kwargs):  # will override apply method
+            kwargs["evaluation"].message(
+                "General",
+                "pyimport",  # see inout.py
+                strip_context(self.get_name()),
+            )
+
         requires = getattr(self, "requires", [])
-
-        for package in requires:
-            try:
-                importlib.import_module(package)
-            except ImportError:
-
-                def apply(**kwargs):  # will override apply method
-                    kwargs["evaluation"].message(
-                        "General",
-                        "pyimport",  # see inout.py
-                        strip_context(self.get_name()),
-                        package,
-                    )
-
-                return apply
-
-        return None
+        return None if check_requires_list(requires) else apply_unavailable
 
     def get_option_string(self, *params):
         s = self.get_option(*params)
@@ -367,8 +479,20 @@ class Builtin(object):
                     return s.get_name()[len(prefix) :], s
         return None, s
 
+    @property
+    def is_literal(self) -> bool:
+        """
+        True if the value can't change, i.e. a value is set and it does not
+        depend on definition bindings. That is why, in contrast to
+        `is_uncertain_final_definitions()` we don't need a `definitions`
+        parameter.
 
-class InstanceableBuiltin(Builtin):
+        Each subclass should decide what is right here.
+        """
+        raise NotImplementedError
+
+
+class BuiltinElement(Builtin, BaseElement):
     def __new__(cls, *args, **kwargs):
         new_kwargs = kwargs.copy()
         new_kwargs["expression"] = False
@@ -391,8 +515,16 @@ class InstanceableBuiltin(Builtin):
     def init(self, *args, **kwargs):
         pass
 
+    def __hash__(self):
+        return hash((self.get_name(), id(self)))
+
 
 class AtomBuiltin(Builtin):
+    """
+    This class is used to define Atoms other than those ones in core, but also
+    have the Builtin function/variable/object properties.
+    """
+
     # allows us to define apply functions, rules, messages, etc. for Atoms
     # which are by default not in the definitions' contribution pipeline.
     # see Image[] for an example of this.
@@ -403,17 +535,17 @@ class AtomBuiltin(Builtin):
 
 
 class Operator(Builtin):
-    operator: typing.Optional[str] = None
-    precedence: typing.Optional[int] = None
+    operator: Optional[str] = None
+    precedence: Optional[int] = None
     precedence_parse = None
     needs_verbatim = False
 
     default_formats = True
 
-    def get_operator(self) -> typing.Optional[str]:
+    def get_operator(self) -> Optional[str]:
         return self.operator
 
-    def get_operator_display(self) -> typing.Optional[str]:
+    def get_operator_display(self) -> Optional[str]:
         if hasattr(self, "operator_display"):
             return self.operator_display
         else:
@@ -421,7 +553,7 @@ class Operator(Builtin):
 
 
 class Predefined(Builtin):
-    def get_functions(self, prefix="apply", is_pymodule=False):
+    def get_functions(self, prefix="apply", is_pymodule=False) -> List[Callable]:
         functions = list(super().get_functions(prefix))
         if prefix == "apply" and hasattr(self, "evaluate"):
             functions.append((Symbol(self.get_name()), self.evaluate))
@@ -429,7 +561,7 @@ class Predefined(Builtin):
 
 
 class SympyObject(Builtin):
-    sympy_name: typing.Optional[str] = None
+    sympy_name: Optional[str] = None
 
     mathics_to_sympy = {}
 
@@ -442,7 +574,7 @@ class SympyObject(Builtin):
     def is_constant(self) -> bool:
         return False
 
-    def get_sympy_names(self) -> typing.List[str]:
+    def get_sympy_names(self) -> List[str]:
         if self.sympy_name:
             return [self.sympy_name]
         return []
@@ -525,25 +657,27 @@ class BinaryOperator(Operator):
 
 
 class Test(Builtin):
-    def apply(self, expr, evaluation) -> Symbol:
+    def apply(self, expr, evaluation) -> Optional[Symbol]:
         "%(name)s[expr_]"
-
-        if self.test(expr):
-            return SymbolTrue
-        else:
-            return SymbolFalse
+        test_expr = self.test(expr)
+        return None if test_expr is None else from_bool(bool(test_expr))
 
 
 class SympyFunction(SympyObject):
-    def apply(self, *args):
-        """
-        Generic apply method that uses the class sympy_name.
-        to call the corresponding sympy function. Arguments are
-        converted to python and the result is converted from sympy
-        """
+    def apply(self, z, evaluation):
+        #
+        # Generic apply method that uses the class sympy_name.
+        # to call the corresponding sympy function. Arguments are
+        # converted to python and the result is converted from sympy
+        #
+        # "%(name)s[z__]"
+        args = z.numerify(evaluation).get_sequence()
         sympy_args = [a.to_sympy() for a in args]
         sympy_fn = getattr(sympy, self.sympy_name)
-        return from_sympy(sympy_fn(*sympy_args))
+        try:
+            return from_sympy(sympy_fn(*sympy_args))
+        except:
+            return
 
     def get_constant(self, precision, evaluation, have_mpmath=False):
         try:
@@ -558,63 +692,126 @@ class SympyFunction(SympyObject):
         else:
             return PrecisionReal(sympy_fn.n(d))
 
-    def get_sympy_function(self, leaves=None):
+    def get_sympy_function(self, elements=None):
         if self.sympy_name:
             return getattr(sympy, self.sympy_name)
         return None
 
-    def prepare_sympy(self, leaves):
-        return leaves
+    def prepare_sympy(self, elements: Iterable) -> Iterable:
+        return elements
 
     def to_sympy(self, expr, **kwargs):
         try:
             if self.sympy_name:
-                leaves = self.prepare_sympy(expr.leaves)
-                sympy_args = [leaf.to_sympy(**kwargs) for leaf in leaves]
+                elements = self.prepare_sympy(expr.elements)
+                sympy_args = [element.to_sympy(**kwargs) for element in elements]
                 if None in sympy_args:
                     return None
-                sympy_function = self.get_sympy_function(leaves)
+                sympy_function = self.get_sympy_function(elements)
                 return sympy_function(*sympy_args)
         except TypeError:
             pass
 
-    def from_sympy(self, sympy_name, leaves):
-        return Expression(self.get_name(), *leaves)
+    def from_sympy(self, sympy_name, elements):
+        return to_expression(self.get_name(), *elements)
 
     def prepare_mathics(self, sympy_expr):
         return sympy_expr
 
 
-class InvalidLevelspecError(Exception):
-    pass
+class BoxExpression(BuiltinElement, BoxElement):
+    # This is the base class for the "Final form"
+    # of formatted expressions.
+    #
+    # The idea is that this class and their subclasses implement
+    # methods of the form ``boxes_to_*`` that now are in ``mathics.core.Expression``.
+    # Also, these objects should not be evaluated, so in the evaluation process should be
+    # considered "inert". However, it could happend that an Expression having them as an element
+    # be evaluable, and try to apply rules. For example,
+    # InputForm[ToBoxes[a+b]]
+    # should be evaluated to ``Expression(SymbolRowBox, '"a"', '"+"', '"b"')``.
+    #
+    # Changes to do, after the refactor of mathics.core:
+    #
 
+    # * Review the implementation
 
-class PartError(Exception):
-    pass
+    attributes = protected | read_protected
 
+    def __new__(cls, *elements, **kwargs):
+        instance = super().__new__(cls, *elements, **kwargs)
+        article = (
+            "an "
+            if instance.get_name()[0].lower() in ("a", "e", "i", "o", "u")
+            else "a "
+        )
 
-class PartDepthError(PartError):
-    def __init__(self, index=0):
-        self.index = index
+        instance.summary_text = (
+            "box representation for "
+            + article
+            + split_name(cls.get_name(short=True)[:-3])
+        )
+        if not instance.__doc__:
+            instance.__doc__ = rf"""
+            <dl>
+            <dt>'{instance.get_name()}'
+            <dd> box structure.
+            </dl>
+            """
 
-
-class PartRangeError(PartError):
-    pass
-
-
-class BoxConstructError(Exception):
-    pass
-
-
-class BoxConstruct(InstanceableBuiltin):
-    def __new__(cls, *leaves, **kwargs):
-        instance = super().__new__(cls, *leaves, **kwargs)
-        instance._leaves = leaves
+        # the __new__ method from BuiltinElement
+        # calls self.init. It is expected that it set
+        # self._elements. However, if it didn't happens,
+        # we set it with a default value.
+        # There should be a better way to implement this
+        # behaviour...
+        if not hasattr(instance, "_elements"):
+            instance._elements = tuple(elements)
         return instance
 
-    def evaluate(self, evaluation):
-        # THINK about: Should we evaluate the leaves here?
-        return
+    @property
+    def is_literal(self) -> bool:
+        """
+        True if the value can't change, i.e. a value is set and it does not
+        depend on definition bindings. That is why, in contrast to
+        `is_uncertain_final_definitions()` we don't need a `definitions`
+        parameter.
+
+        Think about: We will say that a BoxExpression can't change.
+        """
+        return True
+
+    @property
+    def elements(self):
+        return self._elements
+
+    @elements.setter
+    def elements(self, value):
+        self._elements = value
+        return self._elements
+
+    def tex_block(self, tex, only_subsup=False):
+        if len(tex) == 1:
+            return tex
+        else:
+            if not only_subsup or "_" in tex or "^" in tex:
+                return "{%s}" % tex
+            else:
+                return tex
+
+    def to_expression(self) -> Expression:
+        # FIXME: All classes should store their symbol name.
+        # So there should be a self.head.
+        return Expression(Symbol(self.get_name()), *self._elements)
+
+    def replace_vars(self, vars, options=None, in_scoping=True, in_function=True):
+        expr = self.to_expression()
+        result = expr.replace_vars(vars, options, in_scoping, in_function)
+        return result
+
+    # Deprecated: remove eventually
+    def get_elements(self):
+        return self._elements
 
     def get_head_name(self):
         return self.get_name()
@@ -622,6 +819,9 @@ class BoxConstruct(InstanceableBuiltin):
     def get_lookup_name(self):
         return self.get_name()
 
+    def get_sort_key(self) -> tuple:
+        return self.to_expression().get_sort_key()
+
     def get_string_value(self):
         return "-@" + self.get_head_name() + "@-"
 
@@ -629,14 +829,13 @@ class BoxConstruct(InstanceableBuiltin):
         """Mathics SameQ"""
         return expr.sameQ(self)
 
-    def is_atom(self):
-        return False
-
     def do_format(self, evaluation, format):
         return self
 
     def format(self, evaluation, fmt):
-        return self
+        expr = Expression(SymbolHoldForm, self.to_expression())
+        fexpr = expr.format(evaluation, fmt)
+        return fexpr
 
     def get_head(self):
         return Symbol(self.get_name())
@@ -647,57 +846,27 @@ class BoxConstruct(InstanceableBuiltin):
 
     @head.setter
     def head(self, value):
-        raise ValueError("BoxConstruct.head is write protected.")
+        raise ValueError("BoxExpression.head is write protected.")
 
     @property
     def leaves(self):
-        return self._leaves
+        return self.get_elements()
 
     @leaves.setter
     def leaves(self, value):
-        raise ValueError("BoxConstruct.leaves is write protected.")
+        raise ValueError("BoxExpression.elements is write protected.")
 
-    # I need to repeat this, because this is not
-    # an expression...
-    def has_form(self, heads, *leaf_counts):
-        """
-        leaf_counts:
-            (,):        no leaves allowed
-            (None,):    no constraint on number of leaves
-            (n, None):  leaf count >= n
-            (n1, n2, ...):    leaf count in {n1, n2, ...}
-        """
-
-        head_name = self.get_name()
-        if isinstance(heads, (tuple, list, set)):
-            if head_name not in [ensure_context(h) for h in heads]:
-                return False
-        else:
-            if head_name != ensure_context(heads):
-                return False
-        if not leaf_counts:
-            return False
-        if leaf_counts and leaf_counts[0] is not None:
-            count = len(self._leaves)
-            if count not in leaf_counts:
-                if (
-                    len(leaf_counts) == 2
-                    and leaf_counts[1] is None  # noqa
-                    and count >= leaf_counts[0]
-                ):
-                    return True
-                else:
-                    return False
-        return True
-
-    def flatten_pattern_sequence(self, evaluation) -> "BoxConstruct":
+    def flatten_pattern_sequence(self, evaluation) -> "BoxExpression":
         return self
 
-    def get_option_values(self, leaves, **options):
+    def flatten_with_respect_to_head(self, symbol):
+        return self.to_expression().flatten_with_respect_to_head(symbol)
+
+    def get_option_values(self, elements, **options):
         evaluation = options.get("evaluation", None)
         if evaluation:
             default = evaluation.definitions.get_options(self.get_name()).copy()
-            options = Expression("List", *leaves).get_option_values(evaluation)
+            options = ListExpression(*elements).get_option_values(evaluation)
             default.update(options)
         else:
             from mathics.core.parser import parse_builtin_rule
@@ -708,13 +877,13 @@ class BoxConstruct(InstanceableBuiltin):
                 default[option] = parse_builtin_rule(value)
         return default
 
-    def boxes_to_text(self, leaves, **options) -> str:
+    def boxes_to_text(self, elements, **options) -> str:
         raise BoxConstructError
 
-    def boxes_to_mathml(self, leaves, **options) -> str:
+    def boxes_to_mathml(self, elements, **options) -> str:
         raise BoxConstructError
 
-    def boxes_to_tex(self, leaves, **options) -> str:
+    def boxes_to_tex(self, elements, **options) -> str:
         raise BoxConstructError
 
 
@@ -731,19 +900,19 @@ class PatternArgumentError(PatternError):
         super().__init__(name, "argr", count, expected)
 
 
-class PatternObject(InstanceableBuiltin, Pattern):
+class PatternObject(BuiltinElement, Pattern):
     needs_verbatim = True
 
-    arg_counts: typing.List[int] = []
+    arg_counts: List[int] = []
 
     def init(self, expr):
         super().init(expr)
         if self.arg_counts is not None:
-            if len(expr.leaves) not in self.arg_counts:
-                self.error_args(len(expr.leaves), *self.arg_counts)
+            if len(expr.elements) not in self.arg_counts:
+                self.error_args(len(expr.elements), *self.arg_counts)
         self.expr = expr
         self.head = Pattern.create(expr.head)
-        self.leaves = [Pattern.create(leaf) for leaf in expr.leaves]
+        self.elements = [Pattern.create(element) for element in expr.elements]
 
     def error(self, tag, *args):
         raise PatternError(self.get_name(), tag, *args)
@@ -751,31 +920,33 @@ class PatternObject(InstanceableBuiltin, Pattern):
     def error_args(self, count, *expected):
         raise PatternArgumentError(self.get_name(), count, *expected)
 
-    def get_lookup_name(self) -> str:
-        return self.get_name()
+    def get_attributes(self, definitions) -> int:
+        """
+        If self has a head, return head's attributes (an attribute bitmask).
+        Otherwise return an indication that no attributes have been set.
+        """
+        if self.head is None:
+            # FIXME: _Blank in builtin/patterns.py sets head to None.
+            # Figure out if this is the best thing to do and explain why.
+            return no_attributes
+        return self.head.get_attributes(definitions)
 
     def get_head_name(self) -> str:
         return self.get_name()
 
-    def get_sort_key(self, pattern_sort=False):
-        return self.expr.get_sort_key(pattern_sort=pattern_sort)
+    def get_lookup_name(self) -> str:
+        return self.get_name()
+
+    def get_match_candidates(
+        self, elements, expression, attributes, evaluation, vars={}
+    ):
+        return elements
 
     def get_match_count(self, vars={}):
         return (1, 1)
 
-    def get_match_candidates(self, leaves, expression, attributes, evaluation, vars={}):
-        return leaves
-
-    def get_attributes(self, definitions):
-        return self.head.get_attributes(definitions)
-
-
-class MessageException(Exception):
-    def __init__(self, *message):
-        self._message = message
-
-    def message(self, evaluation):
-        evaluation.message(*self._message)
+    def get_sort_key(self, pattern_sort=False) -> tuple:
+        return self.expr.get_sort_key(pattern_sort=pattern_sort)
 
 
 class NegativeIntegerException(Exception):
@@ -795,7 +966,7 @@ class CountableInteger:
     # _support_infinity to False.
     _finite: bool
     _upper_limit: bool
-    _integer: typing.Union[str, int]
+    _integer: Union[str, int]
     _support_infinity = False
 
     def __init__(self, value="Infinity", upper_limit=True):
@@ -849,18 +1020,18 @@ class CountableInteger:
         """
 
         if isinstance(expr, Integer):
-            py_n = expr.get_int_value()
+            py_n = expr.value
             if py_n >= 0:
                 return CountableInteger(py_n, upper_limit=False)
             else:
                 raise NegativeIntegerException()
         elif expr.get_head_name() == "System`UpTo":
-            if len(expr.leaves) != 1:
-                raise MessageException("UpTo", "argx", len(expr.leaves))
+            if len(expr.elements) != 1:
+                raise MessageException("UpTo", "argx", len(expr.elements))
             else:
-                n = expr.leaves[0]
+                n = expr.elements[0]
                 if isinstance(n, Integer):
-                    py_n = n.get_int_value()
+                    py_n = n.value
                     if py_n < 0:
                         raise MessageException("UpTo", "innf", expr)
                     else:
@@ -868,11 +1039,15 @@ class CountableInteger:
                 elif CountableInteger._support_infinity:
                     if (
                         n.get_head_name() == "System`DirectedInfinity"
-                        and len(n.leaves) == 1
+                        and len(n.elements) == 1
                     ):
-                        if n.leaves[0].get_int_value() > 0:
+                        if n.elements[0].get_int_value() > 0:
                             return CountableInteger("Infinity", upper_limit=True)
                         else:
                             return CountableInteger(0, upper_limit=True)
 
         return None  # leave original expression unevaluated
+
+
+# Backward compatibility
+BoxConstruct = BoxExpression
