@@ -6,7 +6,6 @@ There are many different way to specify color; we support all of the color forma
 
 from math import atan2, cos, exp, pi, radians, sin, sqrt
 
-from mathics.version import __version__  # noqa used in loading to check consistency.
 
 from mathics.builtin.colors.color_internals import convert_color
 
@@ -14,18 +13,23 @@ from mathics.builtin.base import (
     Builtin,
     BoxConstructError,
 )
-from mathics.builtin.drawing.graphics_internals import _GraphicsElement, get_class
-from mathics.core.expression import (
-    Expression,
+from mathics.builtin.drawing.graphics_internals import _GraphicsDirective, get_class
+from mathics.core.atoms import (
     Integer,
     Real,
+    MachineReal,
     String,
-    Symbol,
-    SymbolList,
-    from_python,
 )
+from mathics.core.convert.expression import to_expression, to_mathics_list
+from mathics.core.convert.python import from_python
+from mathics.core.element import ImmutableValueMixin
+from mathics.core.expression import Expression
+from mathics.core.list import ListExpression
+from mathics.core.number import machine_epsilon
+from mathics.core.symbols import Symbol
+from mathics.core.systemsymbols import SymbolApply
 
-from mathics.core.numbers import machine_epsilon
+SymbolOpacity = Symbol("Opacity")
 
 
 def _cie2000_distance(lab1, lab2):
@@ -39,15 +43,15 @@ def _cie2000_distance(lab1, lab2):
 
     dL = L2 - L1
     Lm = (L1 + L2) / 2
-    C1 = sqrt(a1 ** 2 + b1 ** 2)
-    C2 = sqrt(a2 ** 2 + b2 ** 2)
+    C1 = sqrt(a1**2 + b1**2)
+    C2 = sqrt(a2**2 + b2**2)
     Cm = (C1 + C2) / 2
 
-    a1 = a1 * (1 + (1 - sqrt(Cm ** 7 / (Cm ** 7 + 25 ** 7))) / 2)
-    a2 = a2 * (1 + (1 - sqrt(Cm ** 7 / (Cm ** 7 + 25 ** 7))) / 2)
+    a1 = a1 * (1 + (1 - sqrt(Cm**7 / (Cm**7 + 25**7))) / 2)
+    a2 = a2 * (1 + (1 - sqrt(Cm**7 / (Cm**7 + 25**7))) / 2)
 
-    C1 = sqrt(a1 ** 2 + b1 ** 2)
-    C2 = sqrt(a2 ** 2 + b2 ** 2)
+    C1 = sqrt(a1**2 + b1**2)
+    C2 = sqrt(a2**2 + b2**2)
     Cm = (C1 + C2) / 2
     dC = C2 - C1
 
@@ -77,8 +81,8 @@ def _cie2000_distance(lab1, lab2):
 
     rT = (
         -2
-        * sqrt(Cm ** 7 / (Cm ** 7 + 25 ** 7))
-        * sin(radians(60 * exp(-((Hm - 275) ** 2 / 25 ** 2))))
+        * sqrt(Cm**7 / (Cm**7 + 25**7))
+        * sin(radians(60 * exp(-((Hm - 275) ** 2 / 25**2))))
     )
     return sqrt(
         (dL / (SL * kL)) ** 2
@@ -97,13 +101,13 @@ def _CMC_distance(lab1, lab2, l, c):
     dL, da, db = L2 - L1, a2 - a1, b2 - b1
     e = machine_epsilon
 
-    C1 = sqrt(a1 ** 2 + b1 ** 2)
-    C2 = sqrt(a2 ** 2 + b2 ** 2)
+    C1 = sqrt(a1**2 + b1**2)
+    C2 = sqrt(a2**2 + b2**2)
 
     h1 = (180 * atan2(b1, a1 + e)) / pi % 360
     dC = C2 - C1
-    dH2 = da ** 2 + db ** 2 - dC ** 2
-    F = C1 ** 2 / sqrt(C1 ** 4 + 1900)
+    dH2 = da**2 + db**2 - dC**2
+    F = C1**2 / sqrt(C1**4 + 1900)
     T = (
         0.56 + abs(0.2 * cos(radians(h1 + 168)))
         if (164 <= h1 and h1 <= 345)
@@ -113,7 +117,7 @@ def _CMC_distance(lab1, lab2, l, c):
     SL = 0.511 if L1 < 16 else (0.040975 * L1) / (1 + 0.01765 * L1)
     SC = (0.0638 * C1) / (1 + 0.0131 * C1) + 0.638
     SH = SC * (F * T + 1 - F)
-    return sqrt((dL / (l * SL)) ** 2 + (dC / (c * SC)) ** 2 + dH2 / SH ** 2)
+    return sqrt((dL / (l * SL)) ** 2 + (dC / (c * SC)) ** 2 + dH2 / SH**2)
 
 
 def _component_distance(a, b, i):
@@ -124,7 +128,41 @@ def _euclidean_distance(a, b):
     return sqrt(sum((x1 - x2) * (x1 - x2) for x1, x2 in zip(a, b)))
 
 
-class _Color(_GraphicsElement):
+class Opacity(_GraphicsDirective):
+    """
+    <dl>
+    <dt>'Opacity[$level$]'
+    <dd> is a graphics directive that sets the opacity to $level$.
+    </dl>
+    >> Graphics[{Blue, Disk[{.5, 1}, 1], Opacity[.4], Red, Disk[], Opacity[.2], Green, Disk[{-.5, 1}, 1]}]
+     = -Graphics-
+    >> Graphics3D[{Blue, Sphere[], Opacity[.4], Red, Cuboid[]}]
+     = -Graphics3D-
+    Notice that 'Opacity' does not overwrite the value of the alpha channel if it is set in a color directive:
+    >> Graphics[{Blue, Disk[], RGBColor[1,0,0,1],Opacity[.2], Rectangle[{0,0},{1,1}]}]
+     = -Graphics-
+    """
+
+    def init(self, item=None, *args, **kwargs):
+        if isinstance(item, (int, float)):
+            item = Expression(SymbolOpacity, MachineReal(item))
+            super(Opacity, self).init(None, item)
+        self.opacity = item.elements[0].to_python()
+
+    def to_css(self):
+        try:
+            if 0.0 <= self.opacity <= 1.0:
+                return self.opacity
+        except:
+            pass
+        return None
+
+    @staticmethod
+    def create_as_style(klass, graphics, item):
+        return klass(item)
+
+
+class _ColorObject(_GraphicsDirective, ImmutableValueMixin):
     formats = {
         # we are adding ImageSizeMultipliers in the rule below, because we do _not_ want color boxes to
         # diminish in size when they appear in lists or rows. we only want the display of colors this
@@ -142,15 +180,14 @@ class _Color(_GraphicsElement):
     default_components = []
 
     def init(self, item=None, components=None):
-        super(_Color, self).init(None, item)
+        super(_ColorObject, self).init(None, item)
         if item is not None:
-            leaves = item.leaves
-            if len(leaves) in self.components_sizes:
+            elements = item.elements
+            if len(elements) in self.components_sizes:
                 # we must not clip here; we copy the components, without clipping,
                 # e.g. RGBColor[-1, 0, 0] stays RGBColor[-1, 0, 0]. this is especially
                 # important for color spaces like LAB that have negative components.
-
-                components = [value.round_to_float() for value in leaves]
+                components = [value.round_to_float() for value in elements]
                 if None in components:
                     raise ColorError
 
@@ -170,7 +207,7 @@ class _Color(_GraphicsElement):
 
     @staticmethod
     def create(expr):
-        head = expr.get_head_name()
+        head = expr.get_head()
         cls = get_class(head)
         if cls is None:
             raise ColorError
@@ -182,7 +219,7 @@ class _Color(_GraphicsElement):
 
     def to_css(self):
         rgba = self.to_rgba()
-        alpha = rgba[3] if len(rgba) > 3 else 1.0
+        alpha = rgba[3] if len(rgba) > 3 else None
         return (
             r"rgb(%f%%, %f%%, %f%%)" % (rgba[0] * 100, rgba[1] * 100, rgba[2] * 100),
             alpha,
@@ -192,7 +229,7 @@ class _Color(_GraphicsElement):
         return self.to_rgba()
 
     def to_expr(self):
-        return Expression(self.get_name(), *self.components)
+        return to_expression(self.get_name(), *self.components)
 
     def to_rgba(self):
         return self.to_color_space("RGB")
@@ -207,7 +244,7 @@ class _Color(_GraphicsElement):
         return components
 
 
-class CMYKColor(_Color):
+class CMYKColor(_ColorObject):
     """
     <dl>
     <dt>'CMYKColor[$c$, $m$, $y$, $k$]'
@@ -260,6 +297,7 @@ class ColorDistance(Builtin):
 
     """
 
+    summary_text = "distance between two colors"
     options = {"DistanceFunction": "Automatic"}
 
     requires = ("numpy",)
@@ -318,8 +356,8 @@ class ColorDistance(Builtin):
                 evaluation.message("ColorDistance", "invdist", distance_function)
                 return
         elif distance_function.has_form("List", 2):
-            if distance_function.leaves[0].get_string_value() == "CMC":
-                if distance_function.leaves[1].get_string_value() == "Acceptability":
+            if distance_function.elements[0].get_string_value() == "CMC":
+                if distance_function.elements[1].get_string_value() == "Acceptability":
                     compute = (
                         lambda c1, c2: _CMC_distance(
                             100 * c1.to_color_space("LAB")[:3],
@@ -329,23 +367,34 @@ class ColorDistance(Builtin):
                         )
                         / 100
                     )
-                elif distance_function.leaves[1].get_string_value() == "Perceptibility":
+                elif (
+                    distance_function.elements[1].get_string_value() == "Perceptibility"
+                ):
                     compute = ColorDistance._distances.get("CMC")
 
-                elif distance_function.leaves[1].has_form("List", 2):
+                elif distance_function.elements[1].has_form("List", 2):
                     if isinstance(
-                        distance_function.leaves[1].leaves[0], Integer
-                    ) and isinstance(distance_function.leaves[1].leaves[1], Integer):
+                        distance_function.elements[1].elements[0], Integer
+                    ) and isinstance(
+                        distance_function.elements[1].elements[1], Integer
+                    ):
                         if (
-                            distance_function.leaves[1].leaves[0].get_int_value() > 0
-                            and distance_function.leaves[1].leaves[1].get_int_value()
+                            distance_function.elements[1].elements[0].get_int_value()
+                            > 0
+                            and distance_function.elements[1]
+                            .elements[1]
+                            .get_int_value()
                             > 0
                         ):
                             lightness = (
-                                distance_function.leaves[1].leaves[0].get_int_value()
+                                distance_function.elements[1]
+                                .elements[0]
+                                .get_int_value()
                             )
                             chroma = (
-                                distance_function.leaves[1].leaves[1].get_int_value()
+                                distance_function.elements[1]
+                                .elements[1]
+                                .get_int_value()
                             )
                             compute = (
                                 lambda c1, c2: _CMC_distance(
@@ -366,15 +415,14 @@ class ColorDistance(Builtin):
 
             def compute(a, b):
                 return Expression(
-                    "Apply",
+                    SymbolApply,
                     distance_function,
-                    Expression(
-                        "List",
-                        Expression(
-                            "List", *[Real(val) for val in a.to_color_space("LAB")]
+                    ListExpression(
+                        to_mathics_list(
+                            *a.to_color_space("LAB"), elements_conversion_fn=Real
                         ),
-                        Expression(
-                            "List", *[Real(val) for val in b.to_color_space("LAB")]
+                        to_mathics_list(
+                            *b.to_color_space("LAB"), elements_conversion_fn=Real
                         ),
                     ),
                 )
@@ -385,8 +433,8 @@ class ColorDistance(Builtin):
 
         def distance(a, b):
             try:
-                py_a = _Color.create(a)
-                py_b = _Color.create(b)
+                py_a = _ColorObject.create(a)
+                py_b = _ColorObject.create(b)
             except ColorError:
                 evaluation.message("ColorDistance", "invarg", a, b)
                 raise
@@ -396,18 +444,17 @@ class ColorDistance(Builtin):
         try:
             if c1.get_head_name() == "System`List":
                 if c2.get_head_name() == "System`List":
-                    if len(c1.leaves) != len(c2.leaves):
+                    if len(c1.elements) != len(c2.elements):
                         evaluation.message("ColorDistance", "invarg", c1, c2)
                         return
                     else:
-                        return Expression(
-                            "List",
-                            *[distance(a, b) for a, b in zip(c1.leaves, c2.leaves)],
+                        return to_mathics_list(
+                            *[distance(a, b) for a, b in zip(c1.elements, c2.elements)],
                         )
                 else:
-                    return Expression(SymbolList, *[distance(c, c2) for c in c1.leaves])
+                    return to_mathics_list(*[distance(c, c2) for c in c1.elements])
             elif c2.get_head_name() == "System`List":
-                return Expression(SymbolList, *[distance(c1, c) for c in c2.leaves])
+                return ListExpression(*[distance(c1, c) for c in c2.elements])
             else:
                 return distance(c1, c2)
         except ColorError:
@@ -421,7 +468,7 @@ class ColorError(BoxConstructError):
     pass
 
 
-class GrayLevel(_Color):
+class GrayLevel(_ColorObject):
     """
     <dl>
     <dt>'GrayLevel[$g$]'
@@ -437,19 +484,22 @@ class GrayLevel(_Color):
     default_components = [0, 1]
 
 
-class Hue(_Color):
+class Hue(_ColorObject):
     """
     <dl>
-    <dt>'Hue[$h$, $s$, $l$, $a$]'
-        <dd>represents the color with hue $h$, saturation $s$,
-        lightness $l$ and opacity $a$.
-    <dt>'Hue[$h$, $s$, $l$]'
-        <dd>is equivalent to 'Hue[$h$, $s$, $l$, 1]'.
-    <dt>'Hue[$h$, $s$]'
-        <dd>is equivalent to 'Hue[$h$, $s$, 1, 1]'.
-    <dt>'Hue[$h$]'
-        <dd>is equivalent to 'Hue[$h$, 1, 1, 1]'.
+      <dt>'Hue[$h$, $s$, $l$, $a$]'
+      <dd>represents the color with hue $h$, saturation $s$, lightness $l$ and opacity $a$.
+
+      <dt>'Hue[$h$, $s$, $l$]'
+      <dd>is equivalent to 'Hue[$h$, $s$, $l$, 1]'.
+
+      <dt>'Hue[$h$, $s$]'
+      <dd>is equivalent to 'Hue[$h$, $s$, 1, 1]'.
+
+      <dt>'Hue[$h$]'
+      <dd>is equivalent to 'Hue[$h$, 1, 1, 1]'.
     </dl>
+
     >> Graphics[Table[{EdgeForm[Gray], Hue[h, s], Disk[{12h, 8s}]}, {h, 0, 1, 1/6}, {s, 0, 1, 1/4}]]
      = -Graphics-
 
@@ -461,7 +511,7 @@ class Hue(_Color):
     components_sizes = [1, 2, 3, 4]
     default_components = [0, 1, 1, 1]
 
-    def hsl_to_rgba(self):
+    def hsl_to_rgba(self) -> tuple:
         h, s, l = self.components[:3]
         if l < 0.5:
             q = l * (1 + s)
@@ -492,7 +542,7 @@ class Hue(_Color):
         return result
 
 
-class LABColor(_Color):
+class LABColor(_ColorObject):
     """
     <dl>
     <dt>'LABColor[$l$, $a$, $b$]'
@@ -506,7 +556,7 @@ class LABColor(_Color):
     default_components = [0, 0, 0, 1]
 
 
-class LCHColor(_Color):
+class LCHColor(_ColorObject):
     """
     <dl>
     <dt>'LCHColor[$l$, $c$, $h$]'
@@ -520,7 +570,7 @@ class LCHColor(_Color):
     default_components = [0, 0, 0, 1]
 
 
-class LUVColor(_Color):
+class LUVColor(_ColorObject):
     """
     <dl>
     <dt>'LCHColor[$l$, $u$, $v$]'
@@ -533,7 +583,7 @@ class LUVColor(_Color):
     default_components = [0, 0, 0, 1]
 
 
-class RGBColor(_Color):
+class RGBColor(_ColorObject):
     """
     <dl>
     <dt>'RGBColor[$r$, $g$, $b$]'
@@ -559,7 +609,7 @@ class RGBColor(_Color):
         return self.components
 
 
-class XYZColor(_Color):
+class XYZColor(_ColorObject):
     """
     <dl>
     <dt>'XYZColor[$x$, $y$, $z$]'
@@ -574,7 +624,7 @@ class XYZColor(_Color):
 
 def expression_to_color(color):
     try:
-        return _Color.create(color)
+        return _ColorObject.create(color)
     except ColorError:
         return None
 
@@ -587,4 +637,4 @@ def color_to_expression(components, colorspace):
     else:
         converted_color_name = colorspace + "Color"
 
-    return Expression(converted_color_name, *components)
+    return to_expression(converted_color_name, *components)
