@@ -1,15 +1,36 @@
 # -*- coding: utf-8 -*-
 
-from mathics.version import __version__  # noqa used in loading to check consistency.
+
 from mathics.builtin.base import BinaryOperator, Predefined, PrefixOperator, Builtin
 from mathics.builtin.lists import InvalidLevelspecError, python_levelspec, walk_levels
-from mathics.core.expression import (
-    Expression,
+from mathics.core.expression import Expression
+
+from mathics.core.attributes import (
+    flat,
+    hold_all,
+    one_identity,
+    orderless,
+    protected,
+    locked,
+)
+
+
+from mathics.core.symbols import (
     Symbol,
     SymbolTrue,
     SymbolFalse,
 )
 
+from mathics.core.systemsymbols import (
+    SymbolAnd,
+    SymbolImplies,
+    SymbolNot,
+    SymbolOr,
+    SymbolXor,
+)
+
+SymbolEquivalent = Symbol("Equivalent")
+
 
 class Or(BinaryOperator):
     """
@@ -30,9 +51,10 @@ class Or(BinaryOperator):
      = a || b
     """
 
+    attributes = flat | hold_all | one_identity | protected
     operator = "||"
     precedence = 215
-    attributes = ("Flat", "HoldAll", "OneIdentity")
+    summary_text = "logic (inclusive) disjunction"
 
     #    rules = {
     #        "Or[a_]": "a",
@@ -43,18 +65,18 @@ class Or(BinaryOperator):
         "Or[args___]"
 
         args = args.get_sequence()
-        leaves = []
+        elements = []
         for arg in args:
             result = arg.evaluate(evaluation)
-            if result.is_true():
+            if result is SymbolTrue:
                 return SymbolTrue
             elif result != SymbolFalse:
-                leaves.append(result)
-        if leaves:
-            if len(leaves) == 1:
-                return leaves[0]
+                elements.append(result)
+        if elements:
+            if len(elements) == 1:
+                return elements[0]
             else:
-                return Expression("Or", *leaves)
+                return Expression(SymbolOr, *elements)
         else:
             return SymbolFalse
 
@@ -78,10 +100,10 @@ class And(BinaryOperator):
      = a && b && c
     """
 
+    attributes = flat | hold_all | one_identity | protected
     operator = "&&"
     precedence = 215
-    attributes = ("Flat", "HoldAll", "OneIdentity")
-
+    summary_text = "logic conjunction"
     #    rules = {
     #        "And[a_]": "a",
     #        "And[a_, a_]": "a",
@@ -92,18 +114,18 @@ class And(BinaryOperator):
         "And[args___]"
 
         args = args.get_sequence()
-        leaves = []
+        elements = []
         for arg in args:
             result = arg.evaluate(evaluation)
-            if result == SymbolFalse:
+            if result is SymbolFalse:
                 return SymbolFalse
-            elif not result.is_true():
-                leaves.append(result)
-        if leaves:
-            if len(leaves) == 1:
-                return leaves[0]
+            elif not result is SymbolTrue:
+                elements.append(result)
+        if elements:
+            if len(elements) == 1:
+                return elements[0]
             else:
-                return Expression("And", *leaves)
+                return Expression(SymbolAnd, *elements)
         else:
             return SymbolTrue
 
@@ -132,6 +154,7 @@ class Not(PrefixOperator):
         "Not[False]": "True",
         "Not[Not[expr_]]": "expr",
     }
+    summary_text = "logic negation"
 
 
 class Nand(Builtin):
@@ -149,6 +172,7 @@ class Nand(Builtin):
     rules = {
         "Nand[expr___]": "Not[And[expr]]",
     }
+    summary_text = "negation of logic conjunction"
 
 
 class Nor(Builtin):
@@ -166,6 +190,7 @@ class Nor(Builtin):
     rules = {
         "Nor[expr___]": "Not[Or[expr]]",
     }
+    summary_text = "negation of logic (inclusive) disjunction"
 
 
 class Implies(BinaryOperator):
@@ -193,17 +218,18 @@ class Implies(BinaryOperator):
     operator = "\u21D2"
     precedence = 200
     grouping = "Right"
+    summary_text = "logic implication"
 
     def apply(self, x, y, evaluation):
         "Implies[x_, y_]"
 
         result0 = x.evaluate(evaluation)
-        if result0 == SymbolFalse:
+        if result0 is SymbolFalse:
             return SymbolTrue
-        elif result0.is_true():
+        elif result0 is SymbolTrue:
             return y.evaluate(evaluation)
         else:
-            return Expression("Implies", result0, y.evaluate(evaluation))
+            return Expression(SymbolImplies, result0, y.evaluate(evaluation))
 
 
 class Equivalent(BinaryOperator):
@@ -232,9 +258,10 @@ class Equivalent(BinaryOperator):
      = True
     """
 
+    attributes = orderless | protected
     operator = "\u29E6"
     precedence = 205
-    attributes = "Orderless"
+    summary_text = "logic equivalence"
 
     def apply(self, args, evaluation):
         "Equivalent[args___]"
@@ -246,17 +273,17 @@ class Equivalent(BinaryOperator):
         flag = False
         for arg in args:
             result = arg.evaluate(evaluation)
-            if result == SymbolFalse or result.is_true():
+            if result is SymbolFalse or result is SymbolTrue:
                 flag = not flag
                 break
         if flag:
             return Expression(
-                "Or",
-                Expression("And", *args),
-                Expression("And", *[Expression("Not", arg) for arg in args]),
+                SymbolOr,
+                Expression(SymbolAnd, *args),
+                Expression(SymbolAnd, *[Expression(SymbolNot, arg) for arg in args]),
             ).evaluate(evaluation)
         else:
-            return Expression("Equivalent", *args)
+            return Expression(SymbolEquivalent, *args)
 
 
 class Xor(BinaryOperator):
@@ -291,32 +318,33 @@ class Xor(BinaryOperator):
      = a \u22BB b
     """
 
+    attributes = flat | one_identity | orderless | protected
     operator = "\u22BB"
     precedence = 215
-    attributes = ("Flat", "OneIdentity", "Orderless")
+    summary_text = "logic (exclusive) disjunction"
 
     def apply(self, args, evaluation):
         "Xor[args___]"
 
         args = args.get_sequence()
-        leaves = []
+        elements = []
         flag = True
         for arg in args:
             result = arg.evaluate(evaluation)
-            if result.is_true():
+            if result is SymbolTrue:
                 flag = not flag
             elif result != SymbolFalse:
-                leaves.append(result)
-        if leaves and flag:
-            if len(leaves) == 1:
-                return leaves[0]
+                elements.append(result)
+        if elements and flag:
+            if len(elements) == 1:
+                return elements[0]
             else:
-                return Expression("Xor", *leaves)
-        elif leaves and not flag:
-            if len(leaves) == 1:
-                return Expression("Not", leaves[0])
+                return Expression(SymbolXor, *elements)
+        elif elements and not flag:
+            if len(elements) == 1:
+                return Expression(SymbolNot, elements[0])
             else:
-                return Expression("Not", Expression("Xor", *leaves))
+                return Expression(SymbolNot, Expression(SymbolXor, *elements))
         else:
             return Symbol(repr(not flag))
 
@@ -329,7 +357,9 @@ class True_(Predefined):
     </dl>
     """
 
+    attributes = locked | protected
     name = "True"
+    summary_text = "boolean constant for True"
 
 
 class False_(Predefined):
@@ -340,7 +370,9 @@ class False_(Predefined):
     </dl>
     """
 
+    attributes = locked | protected
     name = "False"
+    summary_text = "boolean constant for False"
 
 
 class _ShortCircuit(Exception):
@@ -370,7 +402,9 @@ class _ManyTrue(Builtin):
             return
 
         def callback(node):
-            self._short_circuit(Expression(test, node).evaluate(evaluation).is_true())
+            self._short_circuit(
+                Expression(test, node).evaluate(evaluation) is SymbolTrue
+            )
             return node
 
         try:
@@ -402,6 +436,8 @@ class NoneTrue(_ManyTrue):
      = True
     """
 
+    summary_text = "all the elements are False"
+
     def _short_circuit(self, what):
         if what:
             raise _ShortCircuit(SymbolFalse)
@@ -431,6 +467,8 @@ class AnyTrue(_ManyTrue):
      = False
     """
 
+    summary_text = "some of the elements are True"
+
     def _short_circuit(self, what):
         if what:
             raise _ShortCircuit(SymbolTrue)
@@ -460,6 +498,8 @@ class AllTrue(_ManyTrue):
      = True
     """
 
+    summary_text = "all the elements are True"
+
     def _short_circuit(self, what):
         if not what:
             raise _ShortCircuit(SymbolFalse)
