@@ -4,29 +4,44 @@
 Number theoretic functions
 """
 
-import sympy
-from itertools import combinations
-
-from mathics.version import __version__  # noqa used in loading to check consistency.
-from mathics.builtin.base import Builtin, Test, SympyFunction
-from mathics.core.expression import (
-    Expression,
-    Integer,
-    Integer0,
-    Rational,
-    Symbol,
-    from_python,
-    SymbolN,
-)
-from mathics.core.convert import from_sympy, SympyPrime
 import mpmath
+import sympy
+
+from mathics.builtin.base import Builtin, SympyFunction
+from mathics.core.atoms import Integer, Integer0, Integer10, Rational
+from mathics.core.attributes import (
+    listable as A_LISTABLE,
+    numeric_function as A_NUMERIC_FUNCTION,
+    orderless as A_ORDERLESS,
+    protected as A_PROTECTED,
+    read_protected as A_READ_PROTECTED,
+)
+from mathics.core.convert.expression import to_mathics_list
+from mathics.core.convert.python import from_bool, from_python
+from mathics.core.convert.sympy import SympyPrime, from_sympy
+from mathics.core.evaluators import eval_N
+from mathics.core.expression import Expression
+from mathics.core.list import ListExpression
+from mathics.core.symbols import Symbol, SymbolDivide, SymbolFalse
+from mathics.core.systemsymbols import (
+    SymbolCeiling,
+    SymbolComplex,
+    SymbolFloor,
+    SymbolIm,
+    SymbolRe,
+)
+
+SymbolFractionalPart = Symbol("System`FractionalPart")
+SymbolMantissaExponent = Symbol("System`MantissaExponent")
 
 
 class ContinuedFraction(SympyFunction):
     """
+    <url>:Continued fraction: https://en.wikipedia.org/wiki/Continued_fraction</url> (<url>:SymPy: https://docs.sympy.org/latest/modules/ntheory.html#module-sympy.ntheory.continued_fraction</url>, <url>:WMA: https://reference.wolfram.com/language/ref/ContinuedFraction.html</url>)
     <dl>
       <dt>'ContinuedFraction[$x$, $n$]'
-      <dd>generate the first $n$ terms in the continued fraction reprentation of $x$.
+      <dd>generate the first $n$ terms in the continued fraction representation of $x$.
+
       <dt>'ContinuedFraction[$x$]'
       <dd>the complete continued fraction representation for a rational or quadradic irrational number.
     </dl>
@@ -41,13 +56,13 @@ class ContinuedFraction(SympyFunction):
      = {8, {2, 1, 2, 1, 2, 16}}
     """
 
+    attributes = A_LISTABLE | A_NUMERIC_FUNCTION | A_PROTECTED
+    summary_text = "continued fraction expansion"
     sympy_name = "continued_fraction"
 
-    attributes = ("Listable", "NumericFunction")
-
     def apply_1(self, x, evaluation):
         "%(name)s[x_]"
-        return super().apply(x)
+        return super().apply(x, evaluation)
 
     def apply_2(self, x, n, evaluation):
         "%(name)s[x_, n_Integer]"
@@ -57,52 +72,6 @@ class ContinuedFraction(SympyFunction):
         return from_sympy([next(it) for _ in range(py_n)])
 
 
-class CoprimeQ(Builtin):
-    """
-    <dl>
-    <dt>'CoprimeQ[$x$, $y$]'
-        <dd>tests whether $x$ and $y$ are coprime by computing their
-        greatest common divisor.
-    </dl>
-
-    >> CoprimeQ[7, 9]
-     = True
-
-    >> CoprimeQ[-4, 9]
-     = True
-
-    >> CoprimeQ[12, 15]
-     = False
-
-    CoprimeQ also works for complex numbers
-    >> CoprimeQ[1+2I, 1-I]
-     = True
-
-    >> CoprimeQ[4+2I, 6+3I]
-     = True
-
-    >> CoprimeQ[2, 3, 5]
-     = True
-
-    >> CoprimeQ[2, 4, 5]
-     = False
-    """
-
-    attributes = ("Listable",)
-
-    def apply(self, args, evaluation):
-        "CoprimeQ[args__]"
-
-        py_args = [arg.to_python() for arg in args.get_sequence()]
-        if not all(isinstance(i, int) or isinstance(i, complex) for i in py_args):
-            return Symbol("False")
-
-        if all(sympy.gcd(n, m) == 1 for (n, m) in combinations(py_args, 2)):
-            return Symbol("True")
-        else:
-            return Symbol("False")
-
-
 class Divisors(Builtin):
     """
     <dl>
@@ -128,41 +97,17 @@ class Divisors(Builtin):
 
     # TODO: support GaussianIntegers
     # e.g. Divisors[2, GaussianIntegers -> True]
-
-    attributes = ("Listable",)
+    attributes = A_LISTABLE | A_PROTECTED
+    summary_text = "integer divisors"
 
     def apply(self, n, evaluation):
         "Divisors[n_Integer]"
         if n == Integer0:
             return None
-        return Expression(
-            "List", *[from_sympy(i) for i in sympy.divisors(n.to_sympy())]
-        )
+        return to_mathics_list(*sympy.divisors(n.value), elements_conversion_fn=Integer)
 
 
-class EvenQ(Test):
-    """
-    <dl>
-    <dt>'EvenQ[$x$]'
-        <dd>returns 'True' if $x$ is even, and 'False' otherwise.
-    </dl>
-
-    >> EvenQ[4]
-     = True
-    >> EvenQ[-3]
-     = False
-    >> EvenQ[n]
-     = False
-    """
-
-    attributes = ("Listable", "Protected")
-
-    def test(self, n):
-        value = n.get_int_value()
-        return value is not None and value % 2 == 0
-
-
-# FIXME: Previosuly this used gmpy's gcdext. sympy's gcdex is not as powerful
+# FIXME: Previously this used gmpy's gcdext. sympy's gcdex is not as powerful
 # class ExtendedGCD(Builtin):
 #    """
 #    >> ExtendedGCD[10, 15]
@@ -182,7 +127,7 @@ class EvenQ(Test):
 #    'ExtendedGCD' does not work for rational numbers and Gaussian integers yet
 #    """
 #
-#    attributes = ('Listable',)
+#    attributes = A_LISTABLE | A_PROTECTED
 #
 #    def apply(self, ns, evaluation):
 #        'ExtendedGCD[ns___Integer]'
@@ -191,21 +136,62 @@ class EvenQ(Test):
 #        result = 0
 #        coeff = []
 #        for n in ns:
-#            value = n.get_int_value()
+#            value = n.value
 #            if value is None:
 #                return
 #            new_result, c1, c2 = sympy.gcdex(result, value)
 #            result = new_result
 #            coeff = [c * c1 for c in coeff] + [c2]
-#            return Expression('List', Integer(result), Expression(
-#                'List', *(Integer(c) for c in coeff)))
+#            return ListExpression(Integer(result), ListExpression(
+#                *(Integer(c) for c in coeff)))
+
+
+class EulerPhi(SympyFunction):
+    """
+    <url>:Euler's totient function: https://en.wikipedia.org/wiki/Euler%27s_totient_function</url> (<url>:SymPy: https://docs.sympy.org/latest/modules/ntheory.html#sympy.ntheory.factor_.totient</url>, <url>:WMA: https://reference.wolfram.com/language/ref/EulerPhi.html</url>)
+    This function counts positive integers up to $n$ that are relatively prime to $n$.
+    It is typically used in cryptography and in many applications in elementary number theory.
+    <dl>
+      <dt>'EulerPhi[$n$]'
+      <dd>returns the Euler totient function .
+    </dl>
+
+    Compute the Euler totient function:
+    >> EulerPhi[9]
+    = 6
+
+    'EulerPhi' of a negative integer is same as its positive counterpart:
+    >> EulerPhi[-11] == EulerPhi[11]
+    = True
+
+    Large arguments are computed quickly:
+    >> EulerPhi[40!]
+    = 121343746763281707274905415180804423680000000000
+
+    'EulerPhi' threads over lists:
+    >> EulerPhi[Range[1, 17, 2]]
+    = {1, 2, 4, 6, 6, 10, 12, 8, 16}
+    Above, we get consecutive even numbers when the input is prime.
+
+    Compare the results above with:
+    >> EulerPhi[Range[1, 17]]
+    = {1, 1, 2, 2, 4, 2, 6, 4, 6, 4, 10, 4, 12, 6, 8, 8, 16}
+    """
+
+    attributes = A_LISTABLE | A_PROTECTED
+    summary_text = "Euler totient function"
+    sympy_name = "totient"
+
+    def apply(self, n, evaluation):
+        "EulerPhi[n_Integer]"
+        return super().apply(abs(n), evaluation)
 
 
 class FactorInteger(Builtin):
     """
     <dl>
-    <dt>'FactorInteger[$n$]'
-        <dd>returns the factorization of $n$ as a list of factors and exponents.
+      <dt>'FactorInteger[$n$]'
+      <dd>returns the factorization of $n$ as a list of factors and exponents.
     </dl>
 
     >> factors = FactorInteger[2010]
@@ -218,7 +204,8 @@ class FactorInteger(Builtin):
      = {{2, 1}, {3, 1}, {5, 1}, {67, 1}, {2011, -1}}
     """
 
-    attributes = ("Listable", "Protected")
+    attributes = A_LISTABLE | A_PROTECTED
+    summary_text = "list of prime factors and exponents"
 
     # TODO: GausianIntegers option
     # e.g. FactorInteger[5, GaussianIntegers -> True]
@@ -229,8 +216,8 @@ class FactorInteger(Builtin):
         if isinstance(n, Integer):
             factors = sympy.factorint(n.value)
             factors = sorted(factors.items())
-            return Expression(
-                "List", *(Expression("List", factor, exp) for factor, exp in factors)
+            return ListExpression(
+                *(to_mathics_list(factor, exp) for factor, exp in factors)
             )
 
         elif isinstance(n, Rational):
@@ -240,8 +227,8 @@ class FactorInteger(Builtin):
             for factor, exp in factors_denom.items():
                 factors[factor] = factors.get(factor, 0) - exp
             factors = sorted(factors.items())
-            return Expression(
-                "List", *(Expression("List", factor, exp) for factor, exp in factors)
+            return ListExpression(
+                *(to_mathics_list(factor, exp) for factor, exp in factors)
             )
         else:
             return evaluation.message("FactorInteger", "exact", n)
@@ -252,14 +239,14 @@ def _fractional_part(self, n, expr, evaluation):
     if n_sympy.is_constant():
         if n_sympy >= 0:
             positive_integer_part = (
-                Expression("Floor", n).evaluate(evaluation).to_python()
+                Expression(SymbolFloor, n).evaluate(evaluation).to_python()
             )
-            result = n - positive_integer_part
+            result = n - Integer(positive_integer_part)
         else:
             negative_integer_part = (
-                Expression("Ceiling", n).evaluate(evaluation).to_python()
+                Expression(SymbolCeiling, n).evaluate(evaluation).to_python()
             )
-            result = n - negative_integer_part
+            result = n - Integer(negative_integer_part)
     else:
         return expr
 
@@ -295,18 +282,19 @@ class FractionalPart(Builtin):
      = -8769956796 + Pi ^ 20
     """
 
-    attributes = ("Listable", "NumericFunction", "ReadProtected")
+    attributes = A_LISTABLE | A_NUMERIC_FUNCTION | A_READ_PROTECTED | A_PROTECTED
+    summary_text = "fractional part of a number"
 
     def apply(self, n, evaluation):
         "FractionalPart[n_]"
-        expr = Expression("FractionalPart", n)
+        expr = Expression(SymbolFractionalPart, n)
         return _fractional_part(self.__class__.__name__, n, expr, evaluation)
 
     def apply_2(self, n, evaluation):
         "FractionalPart[n_Complex]"
-        expr = Expression("FractionalPart", n)
-        n_real = Expression("Re", n).evaluate(evaluation)
-        n_image = Expression("Im", n).evaluate(evaluation)
+        expr = Expression(SymbolFractionalPart, n)
+        n_real = Expression(SymbolRe, n).evaluate(evaluation)
+        n_image = Expression(SymbolIm, n).evaluate(evaluation)
 
         real_fractional_part = _fractional_part(
             self.__class__.__name__, n_real, expr, evaluation
@@ -314,7 +302,7 @@ class FractionalPart(Builtin):
         image_fractional_part = _fractional_part(
             self.__class__.__name__, n_image, expr, evaluation
         )
-        return Expression("Complex", real_fractional_part, image_fractional_part)
+        return Expression(SymbolComplex, real_fractional_part, image_fractional_part)
 
 
 class FromContinuedFraction(SympyFunction):
@@ -331,130 +319,18 @@ class FromContinuedFraction(SympyFunction):
      = 225 / 157
     """
 
+    attributes = A_NUMERIC_FUNCTION | A_PROTECTED
+
+    summary_text = "reconstructs a number from its continued fraction representation"
     sympy_name = "continued_fraction_reduce"
 
-    attributes = ("NumericFunction",)
-
     def apply_1(self, expr, evaluation):
-        "%(name)s[expr_?ListQ]"
+        "%(name)s[expr_List]"
         nums = expr.to_python()
         if all(isinstance(i, int) for i in nums):
             return from_sympy(sympy.continued_fraction_reduce(nums))
 
 
-class GCD(Builtin):
-    """
-    <dl>
-    <dt>'GCD[$n1$, $n2$, ...]'
-        <dd>computes the greatest common divisor of the given integers.
-    </dl>
-
-    >> GCD[20, 30]
-     = 10
-    >> GCD[10, y]
-     = GCD[10, y]
-
-    'GCD' is 'Listable':
-    >> GCD[4, {10, 11, 12, 13, 14}]
-     = {2, 1, 4, 1, 2}
-
-    'GCD' does not work for rational numbers and Gaussian integers yet.
-    """
-
-    attributes = ("Flat", "Listable", "OneIdentity", "Orderless")
-
-    def apply(self, ns, evaluation):
-        "GCD[ns___Integer]"
-
-        ns = ns.get_sequence()
-        result = 0
-        for n in ns:
-            value = n.get_int_value()
-            if value is None:
-                return
-            result = sympy.gcd(result, value)
-        return Integer(result)
-
-
-class IntegerExponent(Builtin):
-    """
-    <dl>
-    <dt>'IntegerExponent[$n$, $b$]'
-        <dd>gives the highest exponent of $b$ that divides $n$.
-    </dl>
-
-    >> IntegerExponent[16, 2]
-     = 4
-
-    >> IntegerExponent[-510000]
-     = 4
-
-    >> IntegerExponent[10, b]
-     = IntegerExponent[10, b]
-    """
-
-    attributes = ("Listable", "Protected")
-
-    rules = {
-        "IntegerExponent[n_]": "IntegerExponent[n, 10]",
-    }
-
-    messages = {
-        "int": "Integer expected at position 1 in `1`",
-        "ibase": "Base `1` is not an integer greater than 1.",
-    }
-
-    def apply(self, n, b, evaluation):
-        "IntegerExponent[n_Integer, b_Integer]"
-
-        py_n, py_b = n.to_python(), b.to_python()
-        expr = Expression("IntegerExponent", n, b)
-
-        if not isinstance(py_n, int):
-            evaluation.message("IntegerExponent", "int", expr)
-        py_n = abs(py_n)
-
-        if not (isinstance(py_b, int) and py_b > 1):
-            evaluation.message("IntegerExponent", "ibase", b)
-
-        # TODO: Optimise this (dont need to calc. base^result)
-        # NOTE: IntegerExponent[a,b] causes a Python error here when a or b are
-        # symbols
-        result = 1
-        while py_n % (py_b ** result) == 0:
-            result += 1
-
-        return from_python(result - 1)
-
-
-class LCM(Builtin):
-    """
-    <dl>
-    <dt>'LCM[$n1$, $n2$, ...]'
-        <dd>computes the least common multiple of the given integers.
-    </dl>
-
-    >> LCM[15, 20]
-     = 60
-    >> LCM[20, 30, 40, 50]
-     = 600
-    """
-
-    attributes = ("Flat", "Listable", "OneIdentity", "Orderless")
-
-    def apply(self, ns, evaluation):
-        "LCM[ns___Integer]"
-
-        ns = ns.get_sequence()
-        result = 1
-        for n in ns:
-            value = n.get_int_value()
-            if value is None:
-                return
-            result = sympy.lcm(result, value)
-        return Integer(result)
-
-
 class MantissaExponent(Builtin):
     """
     <dl>
@@ -518,37 +394,36 @@ class MantissaExponent(Builtin):
      = {0, 0}
     """
 
-    attributes = ("Listable",)
-
-    rules = {
-        "MantissaExponent[0]": "{0, 0}",
-        "MantissaExponent[0, n_]": "{0, 0}",
-    }
-
+    attributes = A_LISTABLE | A_PROTECTED
     messages = {
         "realx": "The value `1` is not a real number",
         "rbase": "Base `1` is not a real number greater than 1.",
     }
+    rules = {
+        "MantissaExponent[0]": "{0, 0}",
+        "MantissaExponent[0, n_]": "{0, 0}",
+    }
+    summary_text = "decomposes numbers as mantissa and exponent"
 
     def apply(self, n, b, evaluation):
         "MantissaExponent[n_, b_]"
         # Handle Input with special cases such as PI and E
         n_sympy, b_sympy = n.to_sympy(), b.to_sympy()
 
-        expr = Expression("MantissaExponent", n, b)
+        expr = Expression(SymbolMantissaExponent, n, b)
 
         if isinstance(n.to_python(), complex):
             evaluation.message("MantissaExponent", "realx", n)
             return expr
 
         if n_sympy.is_constant():
-            temp_n = Expression(SymbolN, n).evaluate(evaluation)
+            temp_n = eval_N(n, evaluation)
             py_n = temp_n.to_python()
         else:
             return expr
 
         if b_sympy.is_constant():
-            temp_b = Expression(SymbolN, b).evaluate(evaluation)
+            temp_b = eval_N(b, evaluation)
             py_b = temp_b.to_python()
         else:
             return expr
@@ -559,68 +434,38 @@ class MantissaExponent(Builtin):
 
         base_exp = int(mpmath.log(py_n, py_b))
 
-        exp = (base_exp + 1) if base_exp >= 0 else base_exp
-
-        return Expression("List", Expression("Divide", n, b ** exp), exp)
+        exp = Integer((base_exp + 1) if base_exp >= 0 else base_exp)
+        return ListExpression(Expression(SymbolDivide, n, b**exp), exp)
 
     def apply_2(self, n, evaluation):
         "MantissaExponent[n_]"
         n_sympy = n.to_sympy()
-        expr = Expression("MantissaExponent", n)
+        expr = Expression(SymbolMantissaExponent, n)
 
         if isinstance(n.to_python(), complex):
             evaluation.message("MantissaExponent", "realx", n)
             return expr
         # Handle Input with special cases such as PI and E
         if n_sympy.is_constant():
-            temp_n = Expression(SymbolN, n).evaluate(evaluation)
+            temp_n = eval_N(n, evaluation)
             py_n = temp_n.to_python()
         else:
             return expr
 
         base_exp = int(mpmath.log10(py_n))
-        exp = (base_exp + 1) if base_exp >= 0 else base_exp
+        exp = Integer((base_exp + 1) if base_exp >= 0 else base_exp)
 
-        return Expression("List", Expression("Divide", n, (10 ** exp)), exp)
-
-
-class Mod(Builtin):
-    """
-    <dl>
-    <dt>'Mod[$x$, $m$]'
-        <dd>returns $x$ modulo $m$.
-    </dl>
-
-    >> Mod[14, 6]
-     = 2
-    >> Mod[-3, 4]
-     = 1
-    >> Mod[-3, -4]
-     = -3
-    >> Mod[5, 0]
-     : The argument 0 should be nonzero.
-     = Mod[5, 0]
-    """
-
-    attributes = ("Listable", "NumericFunction")
-
-    def apply(self, n, m, evaluation):
-        "Mod[n_Integer, m_Integer]"
-
-        n, m = n.get_int_value(), m.get_int_value()
-        if m == 0:
-            evaluation.message("Mod", "divz", m)
-            return
-        return Integer(n % m)
+        return ListExpression(Expression(SymbolDivide, n, Integer10**exp), exp)
 
 
 class NextPrime(Builtin):
     """
     <dl>
-    <dt>'NextPrime[$n$]'
-        <dd>gives the next prime after $n$.
-    <dt>'NextPrime[$n$,$k$]'
-        <dd>gives the $k$th  prime after $n$.
+      <dt>'NextPrime[$n$]'
+      <dd>gives the next prime after $n$.
+
+      <dt>'NextPrime[$n$,$k$]'
+      <dd>gives the $k$th  prime after $n$.
     </dl>
 
     >> NextPrime[10000]
@@ -645,14 +490,15 @@ class NextPrime(Builtin):
     rules = {
         "NextPrime[n_]": "NextPrime[n, 1]",
     }
+    summary_text = "closest, smallest prime number"
 
     def apply(self, n, k, evaluation):
-        "NextPrime[n_?NumericQ, k_?IntegerQ]"
+        "NextPrime[n_?NumberQ, k_Integer]"
         py_k = k.to_python(n_evaluation=evaluation)
         py_n = n.to_python(n_evaluation=evaluation)
 
         if py_k >= 0:
-            return from_python(sympy.ntheory.nextprime(py_n, py_k))
+            return Integer(sympy.ntheory.nextprime(py_n, py_k))
 
         # Hack to get earlier primes
         result = n.to_python()
@@ -661,29 +507,9 @@ class NextPrime(Builtin):
                 result = sympy.ntheory.prevprime(result)
             except ValueError:
                 # No earlier primes
-                return from_python(-1 * sympy.ntheory.nextprime(0, py_k - i))
+                return Integer(-1 * sympy.ntheory.nextprime(0, py_k - i))
 
-        return from_python(result)
-
-
-class OddQ(Test):
-    """
-    <dl>
-      <dt>'OddQ[$x$]'
-      <dd>returns 'True' if $x$ is odd, and 'False' otherwise.
-    </dl>
-
-    >> OddQ[-3]
-     = True
-    >> OddQ[0]
-     = False
-    """
-
-    attributes = ("Listable", "Protected")
-
-    def test(self, n):
-        value = n.get_int_value()
-        return value is not None and value % 2 != 0
+        return Integer(result)
 
 
 class PartitionsP(SympyFunction):
@@ -697,58 +523,13 @@ class PartitionsP(SympyFunction):
      = {0, 0, 1, 1, 2, 3, 5, 7, 11, 15, 22, 30, 42, 56, 77}
     """
 
-    attributes = ("Listable", "NumericFunction", "Orderless", "Protected")
+    attributes = A_LISTABLE | A_NUMERIC_FUNCTION | A_ORDERLESS | A_PROTECTED
+    summary_text = "number of unrestricted partitions"
     sympy_name = "npartitions"
 
     def apply(self, n, evaluation):
         "PartitionsP[n_Integer]"
-        return super().apply(n)
-
-
-class PowerMod(Builtin):
-    """
-    <dl>
-    <dt>'PowerMod[$x$, $y$, $m$]'
-        <dd>computes $x$^$y$ modulo $m$.
-    </dl>
-
-    >> PowerMod[2, 10000000, 3]
-     = 1
-    >> PowerMod[3, -2, 10]
-     = 9
-    >> PowerMod[0, -1, 2]
-     : 0 is not invertible modulo 2.
-     = PowerMod[0, -1, 2]
-    >> PowerMod[5, 2, 0]
-     : The argument 0 should be nonzero.
-     = PowerMod[5, 2, 0]
-
-    'PowerMod' does not support rational coefficients (roots) yet.
-    """
-
-    attributes = ("Listable",)
-
-    messages = {
-        "ninv": "`1` is not invertible modulo `2`.",
-    }
-
-    def apply(self, a, b, m, evaluation):
-        "PowerMod[a_Integer, b_Integer, m_Integer]"
-
-        a_int = a
-        m_int = m
-        a, b, m = a.value, b.value, m.value
-        if m == 0:
-            evaluation.message("PowerMod", "divz", m)
-            return
-        if b < 0:
-            b = -b
-            try:
-                a = int(sympy.invert(a, m))
-            except sympy.polys.polyerrors.NotInvertible:
-                evaluation.message("PowerMod", "ninv", a_int, m_int)
-                return
-        return Integer(pow(a, b, m))
+        return super().apply(n, evaluation)
 
 
 class Prime(SympyFunction):
@@ -780,7 +561,8 @@ class Prime(SympyFunction):
      = {Prime[0], 2, Prime[1.2], 5}
     """
 
-    attributes = ("Listable", "NumericFunction")
+    attributes = A_LISTABLE | A_NUMERIC_FUNCTION | A_PROTECTED
+    summary_text = "n-esim prime number"
 
     def apply(self, n, evaluation):
         "Prime[n_]"
@@ -788,7 +570,7 @@ class Prime(SympyFunction):
 
     def to_sympy(self, expr, **kwargs):
         if expr.has_form("Prime", 1):
-            return SympyPrime(expr.leaves[0].to_sympy(**kwargs))
+            return SympyPrime(expr.elements[0].to_sympy(**kwargs))
 
 
 class PrimePi(SympyFunction):
@@ -815,17 +597,17 @@ class PrimePi(SympyFunction):
      = 1
     """
 
-    sympy_name = "ntheory.primepi"
+    attributes = A_LISTABLE | A_NUMERIC_FUNCTION | A_PROTECTED
     mpmath_name = "primepi"
-
-    attributes = ("Listable", "NumericFunction")
+    summary_text = "amount of prime numbers less than or equal"
+    sympy_name = "ntheory.primepi"
 
     # TODO: Traditional Form
 
     def apply(self, n, evaluation):
         "PrimePi[n_?NumericQ]"
         result = sympy.ntheory.primepi(n.to_python(n_evaluation=evaluation))
-        return from_python(result)
+        return Integer(result)
 
 
 class PrimePowerQ(Builtin):
@@ -851,11 +633,11 @@ class PrimePowerQ(Builtin):
      = False
     """
 
+    attributes = A_LISTABLE | A_PROTECTED | A_READ_PROTECTED
     rules = {
         "PrimePowerQ[1]": "False",
     }
-
-    attributes = ("Listable", "Protected", "ReadProtected")
+    summary_text = "test if a number is a power of a prime number"
 
     # TODO: GaussianIntegers option
     """
@@ -879,76 +661,23 @@ class PrimePowerQ(Builtin):
         "PrimePowerQ[n_]"
         n = n.get_int_value()
         if n is None:
-            return Symbol("False")
+            return SymbolFalse
 
         n = abs(n)
-        if len(sympy.factorint(n)) == 1:
-            return Symbol("True")
-        else:
-            return Symbol("False")
-
-
-class PrimeQ(SympyFunction):
-    """
-    <dl>
-    <dt>'PrimeQ[$n$]'
-        <dd>returns 'True' if $n$ is a prime number.
-    </dl>
-
-    For very large numbers, 'PrimeQ' uses probabilistic prime testing, so it might be wrong sometimes
-    (a number might be composite even though 'PrimeQ' says it is prime).
-    The algorithm might be changed in the future.
-
-    >> PrimeQ[2]
-     = True
-    >> PrimeQ[-3]
-     = True
-    >> PrimeQ[137]
-     = True
-    >> PrimeQ[2 ^ 127 - 1]
-     = True
-
-    #> PrimeQ[1]
-     = False
-    #> PrimeQ[2 ^ 255 - 1]
-     = False
-
-    All prime numbers between 1 and 100:
-    >> Select[Range[100], PrimeQ]
-     = {2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97}
-
-    'PrimeQ' has attribute 'Listable':
-    >> PrimeQ[Range[20]]
-     = {False, True, True, False, True, False, True, False, False, False, True, False, True, False, False, False, True, False, True, False}
-    """
-
-    attributes = ("Listable", "NumericFunction")
-
-    sympy_name = "isprime"
-
-    def apply(self, n, evaluation):
-        "PrimeQ[n_]"
-
-        n = n.get_int_value()
-        if n is None:
-            return Symbol("False")
-
-        n = abs(n)
-        if sympy.isprime(n):
-            return Symbol("True")
-        else:
-            return Symbol("False")
+        return from_bool(len(sympy.factorint(n)) == 1)
 
 
 class RandomPrime(Builtin):
     """
     <dl>
-    <dt>'RandomPrime[{$imin$, $imax}]'
-        <dd>gives a random prime between $imin$ and $imax$.
-    <dt>'RandomPrime[$imax$]'
-        <dd>gives a random prime between 2 and $imax$.
-    <dt>'RandomPrime[$range$, $n$]'
-        <dd>gives a list of $n$ random primes in $range$.
+      <dt>'RandomPrime[{$imin$, $imax}]'
+      <dd>gives a random prime between $imin$ and $imax$.
+
+      <dt>'RandomPrime[$imax$]'
+      <dd>gives a random prime between 2 and $imax$.
+
+      <dt>'RandomPrime[$range$, $n$]'
+      <dd>gives a list of $n$ random primes in $range$.
     </dl>
 
     >> RandomPrime[{14, 17}]
@@ -989,19 +718,18 @@ class RandomPrime(Builtin):
 
     rules = {
         "RandomPrime[imax_?NotListQ]": "RandomPrime[{1, imax}, 1]",
-        "RandomPrime[int_?ListQ]": "RandomPrime[int, 1]",
-        "RandomPrime[imax_?ListQ, n_?ArrayQ]": (
-            "ConstantArray[RandomPrime[imax, 1], n]"
-        ),
+        "RandomPrime[int_List]": "RandomPrime[int, 1]",
+        "RandomPrime[imax_List, n_?ArrayQ]": ("ConstantArray[RandomPrime[imax, 1], n]"),
         "RandomPrime[imax_?NotListQ, n_?ArrayQ]": (
             "ConstantArray[RandomPrime[{1, imax}, 1], n]"
         ),
     }
+    summary_text = "picks a random prime in an interval"
 
     # TODO: Use random state as in other randomised methods within mathics
 
     def apply(self, interval, n, evaluation):
-        "RandomPrime[interval_?ListQ, n_]"
+        "RandomPrime[interval_List, n_]"
 
         if not isinstance(n, Integer):
             evaluation.message("RandomPrime", "posdim", n)
@@ -1014,105 +742,19 @@ class RandomPrime(Builtin):
 
         imin, imax = min(py_int), max(py_int)
         if imin <= 0 or not isinstance(imin, int):
-            evaluation.message("RandomPrime", "posint", interval.leaves[0])
+            evaluation.message("RandomPrime", "posint", interval.elements[0])
             return
 
         if imax <= 0 or not isinstance(imax, int):
-            evaluation.message("RandomPrime", "posint", interval.leaves[1])
+            evaluation.message("RandomPrime", "posint", interval.elements[1])
             return
 
         try:
             if py_n == 1:
-                return from_python(sympy.ntheory.randprime(imin, imax + 1))
+                return Integer(sympy.ntheory.randprime(imin, imax + 1))
             return from_python(
                 [sympy.ntheory.randprime(imin, imax + 1) for i in range(py_n)]
             )
         except ValueError:
             evaluation.message("RandomPrime", "noprime")
             return
-
-
-class Quotient(Builtin):
-    """
-    <dl>
-    <dt>'Quotient[m, n]'
-      <dd>computes the integer quotient of $m$ and $n$.
-    </dl>
-
-    >> Quotient[23, 7]
-     = 3
-
-    #> Quotient[13, 0]
-     : Infinite expression Quotient[13, 0] encountered.
-     = ComplexInfinity
-    #> Quotient[-17, 7]
-     = -3
-    #> Quotient[-17, -4]
-     = 4
-    #> Quotient[19, -4]
-     = -5
-    """
-
-    attributes = ("Listable", "NumericFunction")
-
-    messages = {
-        "infy": "Infinite expression `1` encountered.",
-    }
-
-    def apply(self, m, n, evaluation):
-        "Quotient[m_Integer, n_Integer]"
-        py_m = m.get_int_value()
-        py_n = n.get_int_value()
-        if py_n == 0:
-            evaluation.message("Quotient", "infy", Expression("Quotient", m, n))
-            return Symbol("ComplexInfinity")
-        return Integer(py_m // py_n)
-
-
-class QuotientRemainder(Builtin):
-    """
-    <dl>
-    <dt>'QuotientRemainder[m, n]'
-      <dd>computes a list of the quotient and remainder from division of $m$ by $n$.
-    </dl>
-
-    >> QuotientRemainder[23, 7]
-     = {3, 2}
-
-    #> QuotientRemainder[13, 0]
-     : The argument 0 in QuotientRemainder[13, 0] should be nonzero.
-     = QuotientRemainder[13, 0]
-    #> QuotientRemainder[-17, 7]
-     = {-3, 4}
-    #> QuotientRemainder[-17, -4]
-     = {4, -1}
-    #> QuotientRemainder[19, -4]
-     = {-5, -1}
-    #> QuotientRemainder[a, 0]
-     = QuotientRemainder[a, 0]
-    #> QuotientRemainder[a, b]
-     = QuotientRemainder[a, b]
-    #> QuotientRemainder[5.2,2.5]
-     = {2, 0.2}
-    #> QuotientRemainder[5, 2.]
-     = {2, 1.}
-    """
-
-    attributes = ("Listable", "NumericFunction")
-
-    messages = {
-        "divz": "The argument 0 in `1` should be nonzero.",
-    }
-
-    def apply(self, m, n, evaluation):
-        "QuotientRemainder[m_, n_]"
-        if m.is_numeric() and n.is_numeric():
-            py_m = m.to_python()
-            py_n = n.to_python()
-            if py_n == 0:
-                return evaluation.message(
-                    "QuotientRemainder", "divz", Expression("QuotientRemainder", m, n)
-                )
-            return Expression("List", Integer(py_m // py_n), (py_m % py_n))
-        else:
-            return Expression("QuotientRemainder", m, n)
