@@ -1,8 +1,8 @@
-#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-import pickle
 from queue import Queue
+import time
+
 
 import os
 import sys
@@ -13,14 +13,42 @@ from typing import Tuple
 from mathics_scanner import TranslateError
 
 from mathics import settings
-from mathics.core.expression import (
-    ensure_context,
-    KeyComparable,
-    SymbolAborted,
-    SymbolList,
+
+from mathics.core.atoms import Integer, String
+from mathics.core.convert.python import from_python
+from mathics.core.element import KeyComparable, ensure_context
+from mathics.core.interrupt import (
+    AbortInterrupt,
+    BreakInterrupt,
+    ContinueInterrupt,
+    ReturnInterrupt,
+    TimeoutInterrupt,
+    WLThrowInterrupt,
+)
+
+from mathics.core.symbols import (
+    Symbol,
     SymbolNull,
 )
 
+from mathics.core.systemsymbols import (
+    SymbolAborted,
+    SymbolBreak,
+    SymbolContinue,
+    SymbolFullForm,
+    SymbolHold,
+    SymbolIn,
+    SymbolMathMLForm,
+    SymbolMessageName,
+    SymbolOut,
+    SymbolOutputForm,
+    SymbolOverflow,
+    SymbolStandardForm,
+    SymbolStringForm,
+    SymbolTeXForm,
+    SymbolThrow,
+)
+
 FORMATS = [
     "StandardForm",
     "FullForm",
@@ -33,36 +61,9 @@ FORMATS = [
     "TableForm",
 ]
 
-
-class EvaluationInterrupt(Exception):
-    pass
-
-
-class AbortInterrupt(EvaluationInterrupt):
-    pass
-
-
-class TimeoutInterrupt(EvaluationInterrupt):
-    pass
-
-
-class ReturnInterrupt(EvaluationInterrupt):
-    def __init__(self, expr):
-        self.expr = expr
-
-
-class BreakInterrupt(EvaluationInterrupt):
-    pass
-
-
-class ContinueInterrupt(EvaluationInterrupt):
-    pass
-
-
-class WLThrowInterrupt(EvaluationInterrupt):
-    def __init__(self, value, tag=None):
-        self.tag = tag
-        self.value = value
+SymbolPre = Symbol("System`$Pre")
+SymbolPrePrint = Symbol("System`$PrePrint")
+SymbolPost = Symbol("System`$Post")
 
 
 def _thread_target(request, queue) -> None:
@@ -203,7 +204,7 @@ class Print(Out):
         }
 
 
-class Result(object):
+class Result:
     def __init__(self, out, result, line_no, last_eval=None) -> None:
         self.out = out
         self.result = result
@@ -218,7 +219,7 @@ class Result(object):
         }
 
 
-class Output(object):
+class Output:
     def max_stored_size(self, settings) -> int:
         return settings.MAX_STORED_SIZE
 
@@ -232,12 +233,11 @@ class Output(object):
         raise NotImplementedError
 
 
-class Evaluation(object):
+class Evaluation:
     def __init__(
         self, definitions=None, output=None, format="text", catch_interrupt=True
     ) -> None:
         from mathics.core.definitions import Definitions
-        from mathics.core.expression import Symbol
 
         if definitions is None:
             definitions = Definitions()
@@ -255,7 +255,6 @@ class Evaluation(object):
         self.quiet_all = False
         self.format = format
         self.catch_interrupt = catch_interrupt
-
         self.SymbolNull = SymbolNull
 
         # status of last evaluate
@@ -264,6 +263,9 @@ class Evaluation(object):
         # Necesary to handle OneIdentity on
         # lhs in assignment
         self.ignore_oneidentity = False
+        # Used in ``mathics.builtin.numbers.constants.get_constant`` and
+        # ``mathics.builtin.numeric.N``.
+        self._preferred_n_method = []
 
     def parse(self, query):
         "Parse a single expression and print the messages."
@@ -301,9 +303,11 @@ class Evaluation(object):
         exception type of result like $Aborted, Overflow, Break, or Continue.
         If none of the above applies self.exc_result is Null
         """
-        from mathics.core.expression import Symbol, Expression
+        from mathics.core.convert.expression import to_expression
+        from mathics.core.expression import Expression
         from mathics.core.rules import Rule
 
+        self.start_time = time.time()
         self.recursion_depth = 0
         self.timeout = False
         self.stopped = False
@@ -325,16 +329,16 @@ class Evaluation(object):
 
         def evaluate():
             if history_length > 0:
-                self.definitions.add_rule("In", Rule(Expression("In", line_no), query))
+                self.definitions.add_rule(
+                    "In", Rule(to_expression("In", line_no), query)
+                )
             if check_io_hook("System`$Pre"):
-                self.last_eval = Expression("System`$Pre", query).evaluate(self)
+                self.last_eval = Expression(SymbolPre, query).evaluate(self)
             else:
                 self.last_eval = query.evaluate(self)
 
             if check_io_hook("System`$Post"):
-                self.last_eval = Expression("System`$Post", self.last_eval).evaluate(
-                    self
-                )
+                self.last_eval = Expression(SymbolPost, self.last_eval).evaluate(self)
             if history_length > 0:
                 if self.predetermined_out is not None:
                     out_result = self.predetermined_out
@@ -344,12 +348,12 @@ class Evaluation(object):
 
                 stored_result = self.get_stored_result(out_result)
                 self.definitions.add_rule(
-                    "Out", Rule(Expression("Out", line_no), stored_result)
+                    "Out", Rule(Expression(SymbolOut, Integer(line_no)), stored_result)
                 )
             if self.last_eval != self.SymbolNull:
                 if check_io_hook("System`$PrePrint"):
                     self.last_eval = Expression(
-                        "System`$PrePrint", self.last_eval
+                        SymbolPrePrint, self.last_eval
                     ).evaluate(self)
                 return self.format_output(self.last_eval, format)
             else:
@@ -371,26 +375,29 @@ class Evaluation(object):
                     or text == "mpq.pow outrageous exp num"  # noqa
                 ):
                     self.message("General", "ovfl")
-                    self.exc_result = Expression("Overflow")
+                    self.exc_result = Expression(SymbolOverflow)
                 else:
                     raise
             except WLThrowInterrupt as ti:
                 if ti.tag:
                     self.exc_result = Expression(
-                        "Hold", Expression("Throw", ti.value, ti.tag)
+                        SymbolHold, Expression(SymbolThrow, ti.value, ti.tag)
                     )
                 else:
-                    self.exc_result = Expression("Hold", Expression("Throw", ti.value))
+                    self.exc_result = Expression(
+                        SymbolHold, Expression(SymbolThrow, ti.value)
+                    )
                 self.message("Throw", "nocatch", self.exc_result)
-            except OverflowError:
-                self.message("General", "ovfl")
-                self.exc_result = Expression("Overflow")
+            #            except OverflowError:
+            #                print("Catch the overflow")
+            #                self.message("General", "ovfl")
+            #                self.exc_result = Expression(SymbolOverflow)
             except BreakInterrupt:
                 self.message("Break", "nofdw")
-                self.exc_result = Expression("Hold", Expression("Break"))
+                self.exc_result = Expression(SymbolHold, Expression(SymbolBreak))
             except ContinueInterrupt:
                 self.message("Continue", "nofdw")
-                self.exc_result = Expression("Hold", Expression("Continue"))
+                self.exc_result = Expression(SymbolHold, Expression(SymbolContinue))
             except TimeoutInterrupt:
                 self.stopped = False
                 self.timeout = True
@@ -415,8 +422,10 @@ class Evaluation(object):
 
         line = line_no - history_length
         while line > 0:
-            unset_in = self.definitions.unset("In", Expression("In", line))
-            unset_out = self.definitions.unset("Out", Expression("Out", line))
+            unset_in = self.definitions.unset("In", Expression(SymbolIn, Integer(line)))
+            unset_out = self.definitions.unset(
+                "Out", Expression(SymbolOut, Integer(line))
+            )
             if not (unset_in or unset_out):
                 break
             line -= 1
@@ -427,7 +436,7 @@ class Evaluation(object):
         that it might have been wrapped in.
         """
         if eval_result.has_form(FORMATS, 1):
-            return eval_result.leaves[0]
+            return eval_result.elements[0]
 
         return eval_result
 
@@ -435,6 +444,8 @@ class Evaluation(object):
         self.stopped = True
 
     def format_output(self, expr, format=None):
+        from mathics.core.formatter import format_element
+
         if format is None:
             format = self.format
 
@@ -444,11 +455,15 @@ class Evaluation(object):
         from mathics.core.expression import Expression, BoxError
 
         if format == "text":
-            result = expr.format(self, "System`OutputForm")
+            result = format_element(expr, self, SymbolOutputForm)
         elif format == "xml":
-            result = Expression("StandardForm", expr).format(self, "System`MathMLForm")
+            result = format_element(
+                Expression(SymbolStandardForm, expr), self, SymbolMathMLForm
+            )
         elif format == "tex":
-            result = Expression("StandardForm", expr).format(self, "System`TeXForm")
+            result = format_element(
+                Expression(SymbolStandardForm, expr), self, SymbolTeXForm
+            )
         elif format == "unformatted":
             self.exc_result = None
             return expr
@@ -456,18 +471,20 @@ class Evaluation(object):
             raise ValueError
 
         try:
+            # With the new implementation, if result is not a ``BoxConstruct``
+            # then we should raise a BoxError here.
             boxes = result.boxes_to_text(evaluation=self)
         except BoxError:
             self.message(
-                "General", "notboxes", Expression("FullForm", result).evaluate(self)
+                "General", "notboxes", Expression(SymbolFullForm, result).evaluate(self)
             )
             boxes = None
         return boxes
 
     def set_quiet_messages(self, messages) -> None:
-        from mathics.core.expression import Expression
+        from mathics.core.list import ListExpression
 
-        value = Expression(SymbolList, *messages)
+        value = ListExpression(*messages)
         self.definitions.set_ownvalue("Internal`$QuietMessages", value)
 
     def get_quiet_messages(self):
@@ -481,17 +498,17 @@ class Evaluation(object):
                 return []
         if not isinstance(value, Expression):
             return []
-        return value.leaves
+        return value.elements
 
     def message(self, symbol, tag, *args) -> None:
-        from mathics.core.expression import String, Symbol, Expression, from_python
+        from mathics.core.expression import Expression
 
         # Allow evaluation.message('MyBuiltin', ...) (assume
         # System`MyBuiltin)
         symbol = ensure_context(symbol)
         quiet_messages = set(self.get_quiet_messages())
 
-        pattern = Expression("MessageName", Symbol(symbol), String(tag))
+        pattern = Expression(SymbolMessageName, Symbol(symbol), String(tag))
 
         if pattern in quiet_messages or self.quiet_all:
             return
@@ -507,7 +524,7 @@ class Evaluation(object):
 
         text = self.definitions.get_value(symbol, "System`Messages", pattern, self)
         if text is None:
-            pattern = Expression("MessageName", Symbol("General"), String(tag))
+            pattern = Expression(SymbolMessageName, Symbol("General"), String(tag))
             text = self.definitions.get_value(
                 "System`General", "System`Messages", pattern, self
             )
@@ -516,16 +533,22 @@ class Evaluation(object):
             text = String("Message %s::%s not found." % (symbol_shortname, tag))
 
         text = self.format_output(
-            Expression("StringForm", text, *(from_python(arg) for arg in args)), "text"
+            Expression(SymbolStringForm, text, *(from_python(arg) for arg in args)),
+            "text",
         )
 
         self.out.append(Message(symbol_shortname, tag, text))
         self.output.out(self.out[-1])
 
     def print_out(self, text) -> None:
-        from mathics.core.expression import from_python
+        from mathics.core.convert.python import from_python
 
-        text = self.format_output(from_python(text), "text")
+        if self.definitions.trace_evaluation:
+            self.definitions.trace_evaluation = False
+            text = self.format_output(from_python(text), "text")
+            self.definitions.trace_evaluation = True
+        else:
+            text = self.format_output(from_python(text), "text")
 
         self.out.append(Print(text))
         self.output.out(self.out[-1])
@@ -547,7 +570,7 @@ class Evaluation(object):
         raise AbortInterrupt
 
     def message_args(self, symbol, given, *needed) -> None:
-        from mathics.core.expression import Symbol
+        from mathics.core.symbols import Symbol
 
         if len(needed) == 1:
             needed = needed[0]
