@@ -4,11 +4,12 @@
 File Stream Operations
 """
 from io import open as io_open
+import os
 import requests
 import sys
 import tempfile
 
-from typing import Optional
+from typing import Optional, Tuple
 
 import os.path as osp
 
@@ -18,7 +19,7 @@ HOME_DIR = osp.expanduser("~")
 PATH_VAR = [".", HOME_DIR, osp.join(ROOT_DIR, "data"), osp.join(ROOT_DIR, "packages")]
 
 
-def create_temporary_file(suffix=None, delete=False):
+def create_temporary_file(prefix="Mathics3-", suffix=None, delete=True):
     if suffix == "":
         suffix = None
 
@@ -38,7 +39,7 @@ def urlsave_tmp(url, location=None, **kwargs):
         try:
             r = requests.get(url, allow_redirects=True)
             if location is None:
-                location = create_temporary_file(suffix=suffix)
+                location = create_temporary_file(prefix="Mathics3-url-", suffix=suffix)
             with open(location, "wb") as fp:
                 fp.write(r.content)
                 result = fp.name
@@ -47,19 +48,28 @@ def urlsave_tmp(url, location=None, **kwargs):
     return result
 
 
-def path_search(filename):
+def path_search(filename: str) -> Tuple[str, bool]:
+    """
+    Search for a Mathics `filename` possibly adding extensions ".mx", or ".m"
+    or as a file under directory PATH_VAR or as an Internet address.
+
+    Return the resolved file name and True if this is a file in the
+    a temporary file created, which happens for Internet addresses,
+    or False if the file is a file in the filesystem.
+    """
     # For names of the form "name`", search for name.mx and name.m
+    is_temporary_file = False
     if filename[-1] == "`":
         filename = filename[:-1].replace("`", osp.sep)
         for ext in [".mx", ".m"]:
-            result = path_search(filename + ext)
+            result, is_temporary_file = path_search(filename + ext)
             if result is not None:
                 filename = None
                 break
     if filename is not None:
         result = None
-        # If filename is an internet address, download the file
-        # and store it in a temporal location
+        # If filename is an Internet address, download the file
+        # and store it in a tempory file
         lenfn = len(filename)
         if (
             (lenfn > 7 and filename[:7] == "http://")
@@ -67,6 +77,7 @@ def path_search(filename):
             or (lenfn > 6 and filename[:6] == "ftp://")
         ):
             result = urlsave_tmp(filename)
+            is_temporary_file = True
         else:
             for p in PATH_VAR + [""]:
                 path = osp.join(p, filename)
@@ -74,74 +85,16 @@ def path_search(filename):
                     result = path
                     break
 
-            # If FindFile resolves to a dir, search within for Kernel/init.m and init.m
+            # If `result` resolves to a dir, search within for Kernel/init.m and init.m
             if result is not None and osp.isdir(result):
                 for ext in [osp.join("Kernel", "init.m"), "init.m"]:
                     tmp = osp.join(result, ext)
                     if osp.isfile(tmp):
-                        return tmp
-    return result
+                        return tmp, is_temporary_file
+    return result, is_temporary_file
 
 
-class StreamsManager(object):
-    __instance = None
-    STREAMS = {}
-
-    @staticmethod
-    def get_instance():
-        """Static access method."""
-        if StreamsManager.__instance == None:
-            StreamsManager()
-        return StreamsManager.__instance
-
-    def __init__(self):
-        """Virtually private constructor."""
-        if StreamsManager.__instance != None:
-            raise Exception("this class is a singleton!")
-        else:
-            StreamsManager.__instance = self
-
-    def add(
-        self,
-        name: str,
-        mode: Optional[str] = None,
-        encoding=None,
-        io=None,
-        num: Optional[int] = None,
-    ) -> Optional["Stream"]:
-        if num is None:
-            num = self.next
-            # In theory in this branch we won't find num.
-        # sanity check num
-        found = self.lookup_stream(num)
-        if found and found is not None:
-            raise Exception(f"Stream {num} already open")
-        stream = Stream(name, mode, encoding, io, num)
-        self.STREAMS[num] = stream
-        return stream
-
-    def delete(self, n: int) -> bool:
-        stream = self.STREAMS.get(n, None)
-        if stream is not None:
-            del self.STREAMS[stream.n]
-            return True
-        return False
-
-    def lookup_stream(self, n=None) -> Optional["Stream"]:
-        if n is None:
-            return None
-        return self.STREAMS.get(n, None)
-
-    @property
-    def next(self):
-        numbers = [stream.n for stream in self.STREAMS.values()] + [2]
-        return max(numbers) + 1
-
-
-stream_manager = StreamsManager()
-
-
-class Stream(object):
+class Stream:
     """
     Opens a stream
 
@@ -150,10 +103,18 @@ class Stream(object):
     with Stream(pypath, "r") as f:
          ...
 
-    However see mathics_open which wraps this
+    However see StreamManager and MathicsOpen which wraps this.
     """
 
-    def __init__(self, name: str, mode="r", encoding=None, io=None, channel_num=None):
+    def __init__(
+        self,
+        name: str,
+        mode="r",
+        encoding=None,
+        io=None,
+        channel_num=None,
+        is_temporary_file: bool = False,
+    ):
         if channel_num is None:
             channel_num = stream_manager.next
         if mode is None:
@@ -163,13 +124,14 @@ class Stream(object):
         self.encoding = encoding
         self.io = io
         self.n = channel_num
+        self.is_temporary_file = is_temporary_file
 
         if mode not in ["r", "w", "a", "rb", "wb", "ab"]:
             raise ValueError("Can't handle mode {0}".format(mode))
 
     def __enter__(self):
         # find path
-        path = path_search(self.name)
+        path, is_temporary_file = path_search(self.name)
         if path is None and self.mode in ["w", "a", "wb", "ab"]:
             path = self.name
         if path is None:
@@ -183,7 +145,13 @@ class Stream(object):
 
         # open the stream
         fp = io_open(path, self.mode, encoding=encoding)
-        stream_manager.add(name=path, mode=self.mode, encoding=encoding, io=fp)
+        stream_manager.add(
+            name=path,
+            mode=self.mode,
+            encoding=encoding,
+            io=fp,
+            is_temporary_file=is_temporary_file,
+        )
         return fp
 
     def __exit__(self, type, value, traceback):
@@ -193,6 +161,77 @@ class Stream(object):
         stream_manager.delete(self.n)
 
 
+class StreamsManager:
+    __instance = None
+    STREAMS = {}
+
+    @staticmethod
+    def get_instance():
+        """Static access method."""
+        if StreamsManager.__instance is None:
+            StreamsManager()
+        return StreamsManager.__instance
+
+    def __init__(self):
+        """Virtually private constructor."""
+        if StreamsManager.__instance is not None:
+            raise Exception("this class is a singleton!")
+        else:
+            StreamsManager.__instance = self
+
+    def add(
+        self,
+        name: str,
+        mode: Optional[str] = None,
+        encoding=None,
+        io=None,
+        num: Optional[int] = None,
+        is_temporary_file: bool = False,
+    ) -> Optional["Stream"]:
+        if num is None:
+            num = self.next
+            # In theory in this branch we won't find num.
+        # sanity check num
+        found = self.lookup_stream(num)
+        if found and found is not None:
+            raise Exception(f"Stream {num} already open")
+        stream = Stream(name, mode, encoding, io, num, is_temporary_file)
+        self.STREAMS[num] = stream
+        return stream
+
+    def delete(self, n: int) -> bool:
+        stream = self.lookup_stream(n)
+        if stream is None:
+            return False
+        self.delete_stream(stream)
+        return True
+
+    def delete_stream(self, stream: Stream):
+        """
+        Delete `stream` from the list of streams we
+        keep track of.
+        """
+        is_temporary_file = stream.is_temporary_file
+        if is_temporary_file:
+            os.unlink(stream.name)
+        del self.STREAMS[stream.n]
+
+    def lookup_stream(self, n: int) -> Optional[Stream]:
+        """
+        Find and return a stream given is stream number `n`.
+        None is returned if no stream found.
+        """
+        return self.STREAMS.get(n, None)
+
+    @property
+    def next(self):
+        numbers = [stream.n for stream in self.STREAMS.values()] + [2]
+        return max(numbers) + 1
+
+
+stream_manager = StreamsManager()
+
+
 stream_manager.add("stdin", mode="r", num=0, io=sys.stdin)
 stream_manager.add("stdout", mode="w", num=1, io=sys.stdout)
 stream_manager.add("stderr", mode="w", num=2, io=sys.stderr)
