@@ -2,27 +2,32 @@ from .helper import evaluate, session
 
 from mathics.builtin.base import BoxConstruct, Predefined
 from mathics.builtin.graphics import GRAPHICS_OPTIONS
+from mathics.core.attributes import hold_all, protected, read_protected
+from mathics.core.expression import Expression
+from mathics.core.symbols import Symbol
+
+SymbolCustomGraphicsBox = Symbol("CustomGraphicsBox")
 
 
 class CustomBoxConstruct(BoxConstruct):
     def __init__(self, evaluation):
         super().__init__(evaluation=evaluation)
-        self._leaves = [1, 2, 3]
+        self._elements = [1, 2, 3]
 
     def boxes_to_text(self, leaves=None, **options):
         if not leaves:
-            leaves = self._leaves
-        return "CustomBoxConstruct<<" + self._leaves.__str__() + ">>"
+            leaves = self.elements
+        return "CustomBoxConstruct<<" + self.elements.__str__() + ">>"
 
     def boxes_to_mathml(self, leaves=None, **options):
         if not leaves:
-            leaves = self._leaves
-        return "CustomBoxConstruct<<" + self._leaves.__str__() + ">>"
+            leaves = self.elements
+        return "CustomBoxConstruct<<" + self.elements.__str__() + ">>"
 
     def boxes_to_tex(self, leaves=None, **options):
         if not leaves:
-            leaves = self._leaves
-        return "CustomBoxConstruct<<" + int(self._leaves) + ">>"
+            leaves = self.elements
+        return "CustomBoxConstruct<<" + int(self.elements) + ">>"
 
 
 class CustomAtom(Predefined):
@@ -44,27 +49,47 @@ class CustomGraphicsBox(BoxConstruct):
     """"""
 
     options = GRAPHICS_OPTIONS
-    attributes = ("HoldAll", "ReadProtected")
+    attributes = hold_all | protected | read_protected
+
+    def init(self, *elems, **options):
+        self._elements = elems
+        self.evaluation = options.pop("evaluation", None)
+        self.box_options = options.copy()
+
+    def to_expression(self):
+        return Expression(SymbolCustomGraphicsBox, *self.elements)
 
     def apply_box(self, elems, evaluation, options):
         """System`MakeBoxes[System`Graphics[elems_, System`OptionsPattern[System`Graphics]],
         System`StandardForm|System`TraditionalForm|System`OutputForm]"""
-        instance = CustomGraphicsBox(*(elems._leaves), evaluation=evaluation)
+        instance = CustomGraphicsBox(*(elems.elements), evaluation=evaluation)
         return instance
 
     def boxes_to_text(self, leaves=None, **options):
         if leaves:
-            self._leaves = leaves
-        return "--custom graphics--: I should plot " + self._leaves.__str__() + " items"
+            self._elements = leaves
+        return (
+            "--custom graphics--: I should plot " + self.elements.__str__() + " items"
+        )
 
     def boxes_to_tex(self, leaves=None, **options):
-        return "--custom graphics--: I should plot " + self._leaves.__str__() + " items"
+        return (
+            "--custom graphics--: I should plot " + self.elements.__str__() + " items"
+        )
 
     def boxes_to_mathml(self, leaves=None, **options):
-        return "--custom graphics--: I should plot " + self._leaves.__str__() + " items"
+        return (
+            "--custom graphics--: I should plot " + self.elements.__str__() + " items"
+        )
 
     def boxes_to_svg(self, evaluation):
-        return "--custom graphics--: I should plot " + self._leaves.__str__() + " items"
+        return (
+            "--custom graphics--: I should plot " + self.elements.__str__() + " items"
+        )
+
+    @property
+    def elements(self):
+        return self._elements
 
 
 def test_custom_boxconstruct():
