@@ -5,13 +5,9 @@ import sys
 
 # FIXME: see if we can refine this better such as
 # by running some Python code and looking for a failure.
-limited_characterset = (
-    sys.platform
-    not in {
-        "win32",
-    }
-    and not os.environ.get("CI")
-)
+limited_characterset = sys.platform not in {
+    "win32",
+} and not os.environ.get("CI")
 if limited_characterset:
 
     def test_non_win32_print():
